<?php
if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class ProductTable extends WP_List_Table{
	/** Class constructor */
	public function __construct() {

		parent::__construct( [
			'singular' => __( 'Product', 'mondialcare' ), //singular name of the listed records
			'plural'   => __( 'Products', 'mondialcare' ), //plural name of the listed records
			'ajax'     => false //does this table support ajax?
		] );
	}


	/**
	 * Retrieve Product data from the database
	 *
	 * @param int $per_page
	 * @param int $page_number
	 *
	 * @return mixed
	 */
	public static function get_product( $per_page = 50, $page_number = 1 ) {
		$array_resultat = [];
		$parent_source = get_option('parent_source');
		$token_type = get_option('token_type');
		$access_token = get_option('access_token');
		
		if ( ! in_array( false, compact( 'parent_source', 'token_type', 'access_token' ) ) ) {
			$request     = Requests::get( $parent_source . '/api/v1/obtenir_produits_par_groupe', [
				'Authorization' => $token_type . ' ' . $access_token
			] );
			$all_product = [];
			if ( $request->status_code === 200 ) {
				$all_product = json_decode( $request->body, true );
			}
//            $array_resultat[ $key ]["shortcode"] = [
//                "[mc_produit id='" . $res->id . "' title='" . $res->title . "']",
//                "[mc_description lang='" . $res->locale . "']",
//                "[mc_pdf id='" . $res->id . "' title='" . $res->title . "']"
//
//
//            ];
			$array_resultat = $all_product;
		}
		return $array_resultat;
	}


	/**
	 * Returns the count of records in the database.
	 *
	 * @return null|string
	 */
	public static function record_count() {
		$array_resultat = [];
		$parent_source = get_option('parent_source');
		$token_type = get_option('token_type');
		$access_token = get_option('access_token');

		if ($parent_source != false && $token_type != false && $access_token != false) {
			$request = Requests::get($parent_source.'/api/v1/obtenir_produits_par_groupe', [
				'Authorization' => $token_type .' '.$access_token
			]);
			if($request->status_code === 200)
			$array_resultat = json_decode($request->body);
			
		}
		return count($array_resultat);
	}


	/** Text displayed when no customer data is available */
	public function no_items() {
		_e( 'Pas de produit disponible pour ce site, Veuillez paramétrer le plugin', 'mondialcare' );
	}


	/**
	 * Render a column when no column specific method exist.
	 *
	 * @param array $item
	 * @param string $column_name
	 *
	 * @return mixed
	 */
	public function column_default( $item, $column_name ) {
		switch ( $column_name ) {
			case 'name':
				return "<h1>". $item["group"][ $column_name ] . "</h1>";
			default:
				return print_r( $item, true ); //Show the whole array for troubleshooting purposes
		}
	}

	/**
	 * Method for shortcode column
	 *
	 * @param array $item an array of DB data
	 *
	 * @return string
	 */
	function column_shortcode( $item ) {
//		foreach ( $item['shortcode'] as $shortcode ) {
//			$output .=  '<span class="shortcode"><input style="margin-bottom: 3px;" type="text"'
//			            . ' onfocus="this.select();" readonly="readonly"'
//			            . ' value="' . esc_attr( $shortcode ) . '"'
//			            . ' class="large-text code" /></span>';
//		}
        $count = 0;
        $output = "";
        foreach ($item["products"] as $product) {
            $open = $count == 0;
            $strOpen = "";
            if ($open) $strOpen = "open";

            ob_start();
            ?>
            <details <?= $strOpen ?>>
                <summary>
                    <svg width="20" height="20" style="box-shadow: 0 0 1px #aeaeae">
                        <image xlink:href=<?= $product['flag'] ?? "" ?> width="20"
                        height="20"/>
                    </svg>
                    <?= $product["nativeLang"] ?>
                </summary>
                <?php
                $shortcodes = [
                    "[mc_produit id='" . $product["id"] . "' title='" . $product["title"] . "']",
                    "[mc_description lang='" . $product["locale"] . "']",
                    "[mc_pdf id='" . $product["id"] . "' title='" . $product["title"] . "']"
                ];
                $o = "";
                foreach ( $shortcodes as $shortcode ) {
                    $o .=  '<span class="shortcode"><input style="margin-bottom: 3px;" type="text"'
                                . ' onfocus="this.select();" readonly="readonly"'
                                . ' value="' . esc_attr( $shortcode ) . '"'
                                . ' class="large-text code" /></span>';
                }
                echo $o;
                ?>
            </details>
            <?php
            $count++;
            $details = ob_get_contents();
            ob_end_clean();
            $output .= $details;
        }
		return $output;
	}



	/**
	 *  Associative array of columns
	 *
	 * @return array
	 */
	function get_columns() {
		$columns = [
			'name' => __( 'Titre', 'mondialcare' ),
			'shortcode' => __( 'Shortcodes', 'mondialcare' )
		];

		return $columns;
	}


	/**
	 * Columns to make sortable.
	 *
	 * @return array
	 */
	public function get_sortable_columns() {
		$sortable_columns = array(
			'name' => array( 'name', true )
		);

		return $sortable_columns;
	}


	/**
	 * Handles data query and filter, sorting, and pagination.
	 */
	public function prepare_items() {

		$this->_column_headers = $this->get_column_info();

		/** Process bulk action */
		$this->process_bulk_action();

		$per_page     = 50;
		$current_page = $this->get_pagenum();
		$total_items  = self::record_count();

		$this->set_pagination_args( [
			'total_items' => $total_items, //WE have to calculate the total number of items
			'per_page'    => $per_page //WE have to determine how many items to show on a page
		] );

		$this->items = self::get_product( $per_page, $current_page );
	}
}