import React, {useEffect} from 'react';
import {Form, Input, Row, Col, DatePicker, Select, Button as AntButton} from 'antd';
import {Container} from './Components/Container.style';
import {CTable, CTbody, CThead, CTheadTitle, CTRow, CTTDBody, TableContainer} from './Components/TableContainer.style';
import {
    PlusOutlined,
    PlusCircleOutlined,
    CheckCircleOutlined,
    UserOutlined,
    CloseCircleOutlined,
} from '@ant-design/icons/lib/icons';
import moment from "moment";
import './App.css';
import {FORMAT_FR} from "./utils/dateUtil";
import localeFr from "antd/es/date-picker/locale/fr_FR";
import {CustomIcon} from "./Components/CustomIcon.style";


const FormAssurer = ({visible_draw, hideDrawer, addData, dataToEdit, label, dataSource, dataSourcePrincipal, onChangeInFamille, showFamille}) => {
    console.log("RENDER: FORM ASSURE");
    let visibility = "block";
    if (!visible_draw) {
        visibility = "none";
    }
    let button;

    if (dataToEdit.length > 0) {
        button = <AntButton htmlType='submit' className="button-add">
            <CheckCircleOutlined style={{verticalAlign: "unset !important"}}/> {label.modifier} </AntButton>
    } else {
        button = <AntButton  htmlType='submit' className="button-add">
            <PlusCircleOutlined style={{verticalAlign: "unset !important"}}/>{label.ajouter}</AntButton>
    }

    const options = [];
    allCountries.forEach((pays, key) => {
        options.push(
            <Select.Option key={key} value={pays["iso2"].toString()}>{pays["name"]}</Select.Option>
        );
    });

    useEffect(() => {
        if (showFamille) {
            if (dataToEdit.length > 0) {
                const datatoEdit = dataToEdit[0];
                const obj_index = dataSourcePrincipal.findIndex((item) => item["personne--indice"] === datatoEdit.indice);

                buildUiFamilleInListAssurer(dataSourcePrincipal[obj_index], obj_index, onChangeInFamille);
            } else {
                buildUiFamilleInListAssurer([], dataSource.length, onChangeInFamille);
            }
        }
    });

    return <Form style={{display: visibility, marginTop: 10}}
                 initialValues={dataToEdit[0]}
                 onFinish={addData}
                 name="basic"
                 labelCol={{
                     span: 12,
                 }}
                 wrapperCol={{
                     span: 24,
                 }}
                 layout='vertical'
    >
        <Row gutter={24}>
            <Col xs={24} md={12}>
                <Form.Item
                    label={label.nom}
                    name="nom"
                    rules={[
                        {
                            required: true,
                            message: label.nom + " " + label.requis,
                        },
                    ]}
                >
                    <Input style={{ textTransform:"uppercase" }}/>
                </Form.Item>
            </Col>
            <Col xs={24} md={12} style={{textAlign: "end"}}>
                <Form.Item
                    label={label.prenom}
                    name="prenom"
                    rules={[
                        {
                            required: true,
                            message: label.prenom + " " + label.requis,
                        },
                    ]}
                >
                    <Input style={{ textTransform:"uppercase" }}/>
                </Form.Item>
            </Col>
        </Row>

        <Row gutter={24}>
            <Col xs={24} md={12}>
                <Form.Item
                    label={label.date_de_naissance}
                    name="date_naiss"
                    rules={[
                        {
                            required: true,
                            message: label.date_de_naissance + " " + label.requis,
                        },
                    ]}
                >
                    <DatePicker
                        style={{width: "100%"}}
                        format={FORMAT_FR}
                        locale={localeFr}
                        required={true}
                        aria-required={true}
                        disabledDate={c => c && c > moment()}/>
                </Form.Item>
            </Col>
            <Col xs={24} md={12} style={{textAlign: "end"}}>
                <Form.Item
                    label={label.num_passeport}
                    name="num_passeport"
                    rules={[
                        {
                            required: true,
                            message: label.num_passeport + " " + label.requis,
                        },
                    ]}
                >
                    <Input style={{ textTransform:"uppercase" }}/>
                </Form.Item>
            </Col>
        </Row>
        <Row gutter={24}>
            <Col xs={24} md={12}>
                <Form.Item
                    label={label.nationalite}
                    name="nationalite"
                    rules={[
                        {
                            required: true,
                            message: label.num_passeport + " " + label.requis,
                        },
                    ]}
                >
                    <Select
                        showSearch={true}
                        filterOption={(input, option) =>
                            option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0
                            || option.props.value.toLowerCase().indexOf(input.toLowerCase()) >= 0
                        }
                        >
                        {options}
                    </Select>
                </Form.Item>
            </Col>
        </Row>

        <Row gutter={24} style={{marginTop: "30px"}}>
            <Col xs={24} className={"widgetFamille"}>

            </Col>
        </Row>

        <Row gutter={24} style={{marginTop: "10px"}}>
            <Col span={12}>
                <AntButton onClick={hideDrawer} className="universal-button"><CloseCircleOutlined/>{label.annuler}
                </AntButton>
            </Col>
            <Col span={12} style={{textAlign: "end"}}>
                {button}
            </Col>
        </Row>

    </Form>
}

const TableAssure = ({dataSource, deleteData, editData, label}) => {
    let rows = [];

    dataSource.forEach((info, index) => {
        var country = allCountries.find((data, key) => {if(data["iso2"] === info.nationalite) return data;})

        rows.push(
            <CTRow className="responsive-table__row" key={index}>
                <CTTDBody className="responsive-table__body__text_mobile">
                    <Row style={{ width: "100%"}}>
                        <Col xs={16}>
                            <h5>{info.nom} {info.prenom}</h5>
                            <Row>
                                <Col xs={9} >
                                    <UserOutlined style={{
                                        width: 55,
                                        height: 55,
                                        borderRadius: "50%",
                                        border: "1px solid #339999",
                                        padding: "4px",
                                        fontSize: 42
                                    }}/>
                                </Col>
                                <Col xs={15} >
                                    <h6>{info.date_naiss.format("DD/MM/YYYY")} - {country["name"]}</h6>
                                    <span style={{fontSize: 12}} className="pc">{label.passeport} : {info.num_passeport}</span>
                                </Col>
                            </Row>
                        </Col>
                        <Col xs={8} style={{margin: "auto"}}>
                            <Row style={{ marginLeft: 2}}>
                                <Col xs={24} onClick={() => editData(info.indice)}>
                                    <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/boucleAssurer/images/edit.png"} style={{
                                    width: 32,
                                    height: 32,
                                    padding: "4px"
                                }} /> {label.modifier} </Col>
                            </Row>
                            <Row style={{marginTop: 5}}>
                                <Col xs={24} onClick={() => deleteData(info.indice)} >
                                    <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/boucleAssurer/images/delete.png"} style={{
                                    width: 32,
                                    height: 32,
                                    padding: "4px"
                                    }}/> {label.buttons.suppression }</Col>
                            </Row>
                        </Col>
                    </Row>
                </CTTDBody>
                <CTTDBody className="responsive-table__body__text responsive-table__body__text--name">
                    <UserOutlined style={{
                        width: 55,
                        height: 55,
                        borderRadius: "50%",
                        border: "1px solid #339999",
                        padding: "4px",
                        fontSize: 42,
                        marginRight: "1rem"
                    }}/>
                    {info.nom} {info.prenom}
                </CTTDBody>
                <CTTDBody className="responsive-table__body__text responsive-table__body__text--birthdate">{info.date_naiss.format("DD/MM/YYYY")}</CTTDBody>
                <CTTDBody className="responsive-table__body__text responsive-table__body__text--passport">{info.num_passeport}</CTTDBody>
                <CTTDBody className="responsive-table__body__text responsive-table__body__text--nationalite">
                    {country["name"]}
                </CTTDBody>
                <CTTDBody className="responsive-table__body__text responsive-table__body__text--actions">
                    <Row style={{ marginLeft: 2}}>
                        <Col xs={24} style={{cursor: "pointer"}} onClick={() => editData(info.indice)}>
                            <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/boucleAssurer/images/edit.png"} style={{
                                width: 32,
                                height: 32,
                                padding: "4px"
                            }} /> {label.modifier} </Col>
                    </Row>
                    <Row style={{marginTop: 5}}>
                        <Col xs={24} style={{cursor: "pointer"}} onClick={() => deleteData(info.indice)} >
                            <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/boucleAssurer/images/delete.png"} style={{
                                width: 32,
                                height: 32,
                                padding: "4px"
                            }}/> {label.buttons.suppression }</Col>
                    </Row>
                </CTTDBody>
            </CTRow>)
    });

    return <TableContainer>
            <CTable>
                <CThead>
                    <CTRow className="responsive-table__row">
                        <CTheadTitle className="responsive-table__head__title--mobile"></CTheadTitle>
                        <CTheadTitle className="responsive-table__head__title--name">{label.nom} et {label.prenom}</CTheadTitle>
                        <CTheadTitle className="responsive-table__head__title--birthdate">{label.date_de_naissance}</CTheadTitle>
                        <CTheadTitle className="responsive-table__head__title--passport">{label.num_passeport}</CTheadTitle>
                        <CTheadTitle className="responsive-table__head__title--nationalite">{label.nationalite}</CTheadTitle>
                        <CTheadTitle className="responsive-table__head__title--actions">{label.actions}</CTheadTitle>
                    </CTRow>
                </CThead>
                <CTbody className="responsive-table__body">
                    {rows}
                </CTbody>
            </CTable>
    </TableContainer>
}

class ListAssurer extends React.Component {
    constructor(props) {
        super(props);
        const personnes = [];
        const data = this.props.data;

        var show_draw = false;
        var show_button = true;
        var show_table = false;

        if ( data["personne"].length > 0 ) {
            for (const index in this.props.data.personne) {
                let personneSource = this.props.data.personne[index];
                const personne = {
                    indice: personneSource["personne--indice"],
                    nom: personneSource["personne--nom"],
                    prenom: personneSource["personne--prenom"],
                    date_naiss: moment(personneSource["personne--date-naiss"], "DD/MM/YYYY"),
                    num_passeport: personneSource["personne--passeport"],
                    nationalite: personneSource["personne--nationalite"]
                }
                personnes.push(personne);
            }
            show_draw = false;
            show_button = true;
            show_table = true;
        }

        this.state = {
            dataSource: personnes,
            famille: props.famille,
            dataToEdit: [],
            visible_draw: show_draw,
            visible_button: show_button,
            visible_table: show_table
        }

        this.addElementInData = this.addElementInData.bind(this);
        this.deleteElementInData = this.deleteElementInData.bind(this);
        this.editElementInData = this.editElementInData.bind(this);
        this.onChangeInFamille = this.onChangeInFamille.bind(this);
    }

    showDrawer = () => {
        this.props.onShowDrawer(true);
        this.setState({
            visible_draw: true,
            visible_button: false,
            visible_table: false,
        });
    }

    hideDrawer = () => {
        this.props.onShowDrawer(false);
        this.setState({
            visible_draw: false,
            visible_button: true,
            visible_table: true,
            dataToEdit: [],
        })
    }

    addData = (values) => {
        const data = this.state.dataSource;
        if (this.state.dataToEdit.length > 0) {
            const dataToEdit = this.state.dataToEdit[0];
            const obj_index = data.findIndex((item) => item.indice === dataToEdit.indice);

            data[obj_index].nom = values.nom;
            data[obj_index].prenom = values.prenom;
            data[obj_index].date_naiss = values.date_naiss;
            data[obj_index].num_passeport = values.num_passeport;
            data[obj_index].nationalite = values.nationalite;


            this.editElementInData(values, dataToEdit.indice);

            this.setState({
                dataSource: data,
                visible_draw: false,
                visible_button: true,
                visible_table: true,
                dataToEdit: [],
            })

        } else {

            let newData = {
                indice: data.length,
                nom: values.nom,
                prenom: values.prenom,
                date_naiss: values.date_naiss,
                num_passeport: values.num_passeport,
                nationalite: values.nationalite
            }

            this.addElementInData(newData);
            this.setState({
                dataSource: [...data, newData],
                visible_draw: false,
                visible_button: true,
                visible_table: true,
                dataToEdit: [],
            })
        }

        this.props.onShowDrawer(false);
    }

    editData = (indice) => {
        const data = this.state.dataSource;
        const dataToEdit = data.filter((item) => item.indice === indice);

        this.setState({
            visible_draw: true,
            visible_button: false,
            visible_table: false,
            dataToEdit: dataToEdit
        });

        this.props.onShowDrawer(true);
    }

    deleteData = (indice) => {
        const data = this.state.dataSource;
        this.setState({
            dataSource: data.filter((item) => item.indice !== indice),
        });

        this.deleteElementInData(indice);
    }

    addElementInData = (info) => {
        const personnes = this.props.data.personne;

        if (personnes[info.indice] === undefined) {
            const newData = {
                "personne--indice": info.indice,
                "personne--nom": info.nom,
                "personne--prenom": info.prenom,
                "personne--date-naiss": moment(info.date_naiss).format("DD/MM/YYYY"),
                "personne--passeport": info.num_passeport,
                "personne--nationalite": info.nationalite,
                "personne--age": moment().diff(info.date_naiss, "years")
            }
            this.props.data.personne.push(newData);
        } else {
            personnes[info.indice]["personne--indice"] = info.indice;
            personnes[info.indice]["personne--nom"] = info.nom;
            personnes[info.indice]["personne--prenom"] = info.prenom;
            personnes[info.indice]["personne--date-naiss"] = moment(info.date_naiss).format("DD/MM/YYYY");
            personnes[info.indice]["personne--passeport"] = info.num_passeport;
            personnes[info.indice]["personne--nationalite"] = info.nationalite;
            personnes[info.indice]["personne--age"] = moment().diff(info.date_naiss, "years");
        }

        this.props.onChange(); // modif de la liste
    }

    editElementInData = (values, indice) => {
        const personnes = this.props.data.personne;
        const obj_index = personnes.findIndex((item) => item["personne--indice"] === indice);

        personnes[obj_index]["personne--nom"] = values.nom;
        personnes[obj_index]["personne--prenom"] = values.prenom;
        personnes[obj_index]["personne--date-naiss"] = moment(values.date_naiss).format("DD/MM/YYYY");
        personnes[obj_index]["personne--passeport"] = values.num_passeport;
        personnes[obj_index]["personne--nationalite"] = values.nationalite;
        personnes[obj_index]["personne--age"] = moment().diff(values.date_naiss, "years");

        this.props.onChange(); // modif de la liste
    }

    deleteElementInData = (indice) => {
        this.props.data.personne = this.props.data.personne.filter(
            (item) => {
                return item["personne--indice"] !== indice
            }
        )

        this.props.onChange(); // modif de la liste
    }

    onChangeInFamille = (personne, indice) => {
        var personnes = this.props.data.personne;
        var info = {};
        for (const pers in personne) {
            info[pers] = personne[pers];
        }
        personnes[indice] = info;
    }

    render() {
        const button_stat = this.state.visible_button;
        const label = window.labels;
        let title;
        let button;
        let table;
        let draw;
        let list = [];
        let famille = this.state.famille;
        if (button_stat) {
            button = <div>
                <AntButton onClick={this.showDrawer} className="button-add" style={{marginTop: 10}}><PlusOutlined/> {label.ajouter_un_assurer}
                </AntButton>
            </div>;
        } else {
            button = "";
        }

        if (this.state.visible_table) {
            table =
                <TableAssure dataSource={this.state.dataSource} deleteData={this.deleteData} editData={this.editData}
                             label={label}/>
        } else {
            table = "";
        }

        if (this.state.visible_draw) {
            draw = <FormAssurer
                visible_draw={this.state.visible_draw}
                hideDrawer={this.hideDrawer}
                addData={this.addData}
                dataToEdit={this.state.dataToEdit}
                label={label}
                dataSource={this.state.dataSource}
                dataSourcePrincipal={this.props.data.personne}
                onChangeInFamille={this.onChangeInFamille}
                showFamille={famille}
            >
            </FormAssurer>
        } else {
            draw = "";
        }

        if (this.state.dataToEdit.length > 0) {
            let pers = this.state.dataToEdit[0];
            title = <h3 style={{textAlign: "center"}}>{label.information_de_l_assurer} {pers.indice + 1}</h3>
        } else {
            title = <h3 style={{textAlign: "center"}}> {label.les_personnes_a_assurer} </h3>
        }

        if (this.props.data.personne.length > 0) {
            this.props.data.personne.forEach((info, index) => {
                let information = <>
                    <Input type="hidden" name={"personne[" + index + "][personne--indice]"}
                           value={info["personne--indice"]}/>
                    <Input type="hidden" name={"personne[" + index + "][personne--nom]"} value={info["personne--nom"]}/>
                    <Input type="hidden" name={"personne[" + index + "][personne--prenom]"}
                           value={info["personne--prenom"]}/>
                    <Input type="hidden" name={"personne[" + index + "][personne--date-naiss]"}
                           value={info["personne--date-naiss"]}/>
                    <Input type="hidden" name={"personne[" + index + "][personne--passeport]"}
                           value={info["personne--passeport"]}/>
                    <Input type="hidden" name={"personne[" + index + "][personne--nationalite]"}
                           value={info["personne--nationalite"]}/>
                    <Input type="hidden" name={"personne[" + index + "][personne--age]"}
                           value={info["personne--age"]}/>
                </>;
                list.push(information);

                if (famille) {
                    let infoFamille = <>
                        <Input type="hidden" name={"personne[" + index + "][personne--conjoint-nom]"}
                               value={info["personne--conjoint-nom"]}/>
                        <Input type="hidden" name={"personne[" + index + "][personne--conjoint-prenom]"}
                               value={info["personne--conjoint-prenom"]}/>
                        <Input type="hidden" name={"personne[" + index + "][personne--conjoint-datenais]"}
                               value={info["personne--conjoint-datenais"]}/>
                        <Input type="hidden" name={"personne[" + index + "][personne--conjoint-nationalite]"}
                               value={info["personne--conjoint-nationalite"]}/>
                        <Input type="hidden" name={"personne[" + index + "][personne--conjoint-passeport]"}
                               value={info["personne--conjoint-passeport"]}/>
                        <Input type="hidden" name={"personne[" + index + "][personne--enfants]"}
                               value={info["personne--enfants"]}/>
                    </>;
                    list.push(infoFamille);
                }
            })
        }

        return (
            <Container id="list_assure">
                <div>{title}</div>
                <div>{table}</div>
                <div style={{textAlign: "center"}}>{button}</div>
                <div>{draw}</div>
                <div>{list}</div>
            </Container>
        )
    }
}

function App({data, famille, onChange, onShowDrawer}) {
    return (
        <ListAssurer data={data} famille={famille} onChange={onChange} onShowDrawer={onShowDrawer}/>
    );
}

export default App;


