import React, { Component } from "react";
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faTrash } from '@fortawesome/free-solid-svg-icons';
import { Input, DatePicker, Row, Col, Tooltip } from 'antd';
import moment from 'moment';
import 'moment/locale/fr';
import { FORMAT_FR } from '../utils/dateUtil';
import localeFr from 'antd/es/date-picker/locale/fr_FR';
moment().locale("fr");
class Conjoint extends Component {
    constructor(props) {
        super(props);
        this.onConjointChange = this.onConjointChange.bind(this);
        this.updateNom = this.updateNom.bind(this);
        this.updatePrenom = this.updatePrenom.bind(this);
        this.updateDateNaiss = this.updateDateNaiss.bind(this);
        this.onConjointChange = this.onConjointChange.bind(this);
        this.onDeleteConjoint = this.onDeleteConjoint.bind(this);
    }
    componentDidMount() {
    }
    render() {
        const { indice, layout } = this.props;
        const {
            "nom": enom,
            "prenom": eprenom,
            "dateNaiss": edateNaiss
        } = this.props.conjoint;
        const labels = window.labels;
        const labelEntite = labels.conjoint; //array
        const edateNaissObject = edateNaiss ? moment(edateNaiss, "DD/MM/YYYY") : "";
        const nomKey = "conjoint-nom";
        const prenomKey = "conjoint-prenom";
        const datenaissKey = "conjoint-datenaiss";
        if (layout && layout.toLowerCase() === "desktop")
            return (
                <tr className="conjoint">
                    <td>
                        <Tooltip title={labelEntite[nomKey] ? labelEntite[nomKey] : "Nom"}>
                            <Input
                                onChange={this.updateNom}
                                value={enom || ""}
                                required={true}
                                aria-required={true}
                                placeholder={labelEntite[nomKey] ? labelEntite[nomKey] : "Nom"} />
                        </Tooltip>
                    </td>
                    <td>
                        <Tooltip title={labelEntite[prenomKey] ? labelEntite[prenomKey] : "Prenom"}>
                            <Input
                                onChange={this.updatePrenom}
                                value={eprenom || ""}
                                required={true}
                                aria-required={true}
                                placeholder={labelEntite[prenomKey] ? labelEntite[prenomKey] : "Prenom"}
                            />
                        </Tooltip>
                    </td>
                    <td>
                        <Tooltip title={labelEntite[datenaissKey] ? labelEntite[datenaissKey] : "Date de naissance"}>
                            <DatePicker
                                format={FORMAT_FR}
                                locale={localeFr}
                                required={true}
                                aria-required={true}
                                disabledDate={c => c && c > moment()}
                                placeholder={labelEntite[datenaissKey] ? labelEntite[datenaissKey] : "Date de naissance"}
                                onChange={this.updateDateNaiss} value={edateNaissObject} />
                        </Tooltip>
                    </td>
                    <td>
                        <button
                            type="button"
                            className="btn btn-error button-secondary"
                            style={{ padding: '0 1rem' }}
                            onClick={this.onDeleteConjoint}
                            title={labels.buttons.suppression}>
                            <FontAwesomeIcon icon={faTrash} />
                        </button>
                    </td>
                </tr>
            );
        else if (layout && layout.toLowerCase() === "mobile") {
            return (
                <Row className='conjoint' gutter={[8, 16]}>
                    <Col span={24}>
                        {labelEntite["conjoint-num"] ? labelEntite["conjoint-num"].replace("*num*", indice + 1) : "Conjoint n°" + (indice + 1)}
                    </Col>
                    <Col span={24}>
                        <Tooltip title={labelEntite[nomKey] ? labelEntite[nomKey] : "Nom"}>
                            <Input
                                onChange={this.updateNom}
                                value={enom || ""}
                                required={true}
                                aria-required={true}
                                placeholder={labelEntite[nomKey] ? labelEntite[nomKey] : "Nom"}
                            />
                        </Tooltip>
                    </Col>
                    <Col span={24}>
                        <Tooltip title={labelEntite[prenomKey] ? labelEntite[prenomKey] : "Prenom"}>
                            <Input
                                onChange={this.updatePrenom}
                                value={eprenom || ""}
                                required={true}
                                aria-required={true}
                                placeholder={labelEntite[prenomKey] ? labelEntite[prenomKey] : "Prenom"}
                            />
                        </Tooltip>
                    </Col>
                    <Col>
                        <Tooltip title={labelEntite[datenaissKey] ? labelEntite[datenaissKey] : "Date de naissance"}>
                            <DatePicker
                                format={FORMAT_FR}
                                locale={localeFr}
                                showToday={false}
                                inputReadOnly={false}
                                required={true}
                                aria-required={true}
                                disabledDate={c => c && c > moment()}
                                placeholder={labelEntite[datenaissKey] ? labelEntite[datenaissKey] : "Sa date de naissance"}
                                onChange={this.updateDateNaiss} value={edateNaissObject} />
                        </Tooltip>
                    </Col>
                    <Col>
                        <button type="button" className="btn btn-error button-secondary" onClick={this.onDeleteConjoint} title={labels.buttons.suppression}>
                            <FontAwesomeIcon icon={faTrash} />
                        </button>
                    </Col>
                </Row>
            );
        }
    }
    onConjointChange(conjoint, indice) {
        this.props.onConjointChange(conjoint, indice);
    }
    updateNom(e) {
        const { conjoint } = this.props;
        conjoint["nom"] = e.target.value;
        this.onConjointChange(conjoint, this.props.indice);
    }
    updatePrenom(e) {
        const { conjoint } = this.props;
        conjoint["prenom"] = e.target.value;
        this.onConjointChange(conjoint, this.props.indice);
    }
    updateDateNaiss(date, dateString) {
        const { conjoint } = this.props;
        conjoint["dateNaiss"] = dateString;
        this.onConjointChange(conjoint, this.props.indice);
    }
    onDeleteConjoint(e) {
        this.props.onDeleteConjoint(this.props.indice);
    }
}
export default Conjoint;