import React, { Component } from "react";
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faTrash } from '@fortawesome/free-solid-svg-icons';
import { Input, DatePicker, Row, Col, Tooltip, Popconfirm, message } from 'antd';
import moment from 'moment';
import 'moment/locale/fr';
import { disabledFutureDate, FORMAT_FR } from '../utils/dateUtil';
import localeFr from 'antd/es/date-picker/locale/fr_FR';
moment().locale("fr");
class Enfant extends Component {
    state = {
        popupVisible: false,
        condition: false
    }
    constructor(props) {
        super(props);
        this.onEnfantChange = this.onEnfantChange.bind(this);
        this.updateNom = this.updateNom.bind(this);
        this.updatePrenom = this.updatePrenom.bind(this);
        this.updateDateNaiss = this.updateDateNaiss.bind(this);
        this.onEnfantChange = this.onEnfantChange.bind(this);
        this.onDeleteEnfant = this.onDeleteEnfant.bind(this);

        this.handlePopupVisibilityChange = this.handlePopupVisibilityChange.bind(this);
    }
    componentDidMount() {
    }
    render() {
        const { indice, layout, enfantMaxAge, popupQuestion, popupOkText, popupCancelText } = this.props;
        const { popupVisible } = this.state;
        const {
            "nom": enom,
            "prenom": eprenom,
            "dateNaiss": edateNaiss
        } = this.props.enfant;
        const labels = window.labels;
        const edateNaissObject = edateNaiss ? moment(edateNaiss, "DD/MM/YYYY") : "";
        if (layout && layout.toLowerCase() === "desktop")
            return (
                <tr className="enfant">
                    <td>
                        <Tooltip title={labels.enfants["enfant-nom"] ? labels.enfants["enfant-nom"] : "Nom"}>
                            <Input
                                onChange={this.updateNom}
                                value={enom || ""}
                                required={true}
                                aria-required={true}
                                placeholder={labels.enfants["enfant-nom"] ? labels.enfants["enfant-nom"] : "Nom"} />
                        </Tooltip>
                    </td>
                    <td>
                        <Tooltip title={labels.enfants["enfant-prenom"] ? labels.enfants["enfant-prenom"] : "Prenom"}>
                            <Input
                                onChange={this.updatePrenom}
                                value={eprenom || ""}
                                required={true}
                                aria-required={true}
                                placeholder={labels.enfants["enfant-prenom"] ? labels.enfants["enfant-prenom"] : "Prenom"}
                            />
                        </Tooltip>
                    </td>
                    <td>
                        <Tooltip title={labels.enfants["enfant-datenaiss"] ? labels.enfants["enfant-datenaiss"] : "Date de naissance"}>
                            <Popconfirm
                                title={popupQuestion}
                                visible={popupVisible}
                                onVisibleChange={this.handlePopupVisibilityChange}
                                onConfirm={this.confirm.bind(this)}
                                onCancel={this.cancel.bind(this)}
                                okText={popupOkText}
                                cancelText={popupCancelText}
                            >
                                <DatePicker
                                    disabledDate={current =>  disabledFutureDate(current, enfantMaxAge)}
                                    format={FORMAT_FR}
                                    locale={localeFr}
                                    required={true}
                                    aria-required={true}
                                    placeholder={labels.enfants["enfant-datenaiss"] ? labels.enfants["enfant-datenaiss"] : "Date de naissance"}
                                    onChange={this.updateDateNaiss} value={edateNaissObject} />
                            </Popconfirm>
                        </Tooltip>
                    </td>
                    <td>
                        <button
                            type="button"
                            className="btn btn-error button-secondary"
                            style={{ padding: '0 1rem' }}
                            onClick={this.onDeleteEnfant}
                            title={labels.buttons.suppression}>
                            <FontAwesomeIcon icon={faTrash} />
                        </button>
                    </td>
                </tr>
            );
        else if (layout && layout.toLowerCase() === "mobile") {
            return (
                <Row className='enfant' gutter={[8, 16]}>
                    <Col span={24}>
                        {labels.enfants["enfant-num"] ? labels.enfants["enfant-num"].replace("*num*", indice + 1) : "Enfant n°" + (indice + 1)}
                    </Col>
                    <Col span={24}>
                        <Tooltip title={labels.enfants["enfant-nom"] ? labels.enfants["enfant-nom"] : "Nom"}>
                            <Input
                                onChange={this.updateNom}
                                value={enom || ""}
                                required={true}
                                aria-required={true}
                                placeholder={labels.enfants["enfant-nom"] ? labels.enfants["enfant-nom"] : "Nom"}
                            />
                        </Tooltip>
                    </Col>
                    <Col span={24}>
                        <Tooltip title={labels.enfants["enfant-prenom"] ? labels.enfants["enfant-prenom"] : "Prenom"}>
                            <Input
                                onChange={this.updatePrenom}
                                value={eprenom || ""}
                                required={true}
                                aria-required={true}
                                placeholder={labels.enfants["enfant-prenom"] ? labels.enfants["enfant-prenom"] : "Prenom"}
                            />
                        </Tooltip>
                    </Col>
                    <Col>
                        <Tooltip title={labels.enfants["enfant-datenaiss"] ? labels.enfants["enfant-datenaiss"] : "Date de naissance"}>
                            <Popconfirm
                                title={popupQuestion}
                                visible={popupVisible}
                                onVisibleChange={this.handlePopupVisibilityChange}
                                onConfirm={this.confirm.bind(this)}
                                onCancel={this.cancel.bind(this)}
                                okText={popupOkText}
                                cancelText={popupCancelText}
                            >
                                <DatePicker
                                    disabledDate={current => disabledFutureDate(current, enfantMaxAge)}
                                    format={FORMAT_FR}
                                    locale={localeFr}
                                    showToday={false}
                                    required={true}
                                    aria-required={true}
                                    inputReadOnly={false}
                                    placeholder={labels.enfants["enfant-datenaiss"] ? labels.enfants["enfant-datenaiss"] : "Sa date de naissance"}
                                    onChange={this.updateDateNaiss} value={edateNaissObject} />
                            </Popconfirm>
                        </Tooltip>
                    </Col>
                    <Col>
                        <button type="button" className="btn btn-error button-secondary" onClick={this.onDeleteEnfant} title={labels.buttons.suppression}>
                            <FontAwesomeIcon icon={faTrash} />
                        </button>
                    </Col>
                </Row>
            );
        }
    }
    confirm = () => {
        this.setState({ popupVisible: false });
        message.success(this.props.okText);
    };

    cancel = () => {
        this.setState({ popupVisible: false });
        message.error(this.props.cancelText);
        this.updateDateNaiss(null, '');
    };
    handlePopupVisibilityChange(visible) {
        if (!visible) {
            this.setState({
                visible//object
            });
            return;
        }
        // Determining condition before show the popconfirm.
        if (this.state.condition) {
            //On accepte
        } else {
            this.setState({ visible }) // show the popconfirm
        }
    }
    onEnfantChange(enfant, indice) {
        this.props.onEnfantChange(enfant, indice);
    }
    updateNom(e) {
        const { enfant } = this.props;
        enfant["nom"] = e.target.value;
        this.onEnfantChange(enfant, this.props.indice);
    }
    updatePrenom(e) {
        const { enfant } = this.props;
        enfant["prenom"] = e.target.value;
        this.onEnfantChange(enfant, this.props.indice);
    }
    updateDateNaiss(date, dateString) {
        const { enfant, popupAgeInterval } = this.props;
        const inRange = (x, min, max) => { // return true if in range, otherwise false
            return ((x - min) * (x - max) <= 0);
        }

        const age = moment().diff(date, 'years');
        this.setState({
            condition: inRange(age, popupAgeInterval[0], popupAgeInterval[1]),
            popupVisible: inRange(age, popupAgeInterval[0], popupAgeInterval[1])
        });
        enfant["dateNaiss"] = dateString;
        this.onEnfantChange(enfant, this.props.indice);
    }
    onDeleteEnfant(e) {
        this.props.onDeleteEnfant(this.props.indice);
    }
}
export default Enfant;