import React from 'react';
import ReactDOM from 'react-dom';
import App from './App';
import {ConfigProvider} from "antd";
import Personne from "./personne/personne";
import frLocale from 'antd/es/date-picker/locale/fr_FR';
import enLocale from 'antd/es/date-picker/locale/en_US';
import deLocale from 'antd/es/date-picker/locale/de_DE';
import esLocale from 'antd/es/date-picker/locale/es_ES';
import nlLocale from 'antd/es/date-picker/locale/nl_NL';
import itLocale from 'antd/es/date-picker/locale/it_IT';
import brLocale from 'antd/es/date-picker/locale/pt_BR';

const locales = {
  'fr': { localeObj: frLocale, momentLocale: 'fr' },
  'en': { localeObj: enLocale, momentLocale: 'en-gb' },
  'de': { localeObj: deLocale, momentLocale: 'de' },
  'es': { localeObj: esLocale, momentLocale: 'es' },
  'nl': { localeObj: nlLocale, momentLocale: 'nl' },
  'it': { localeObj: itLocale, momentLocale: 'it' },
  'br': { localeObj: brLocale, momentLocale: 'br' },
};

window.boucleAssurer = function( data, famille, onChange, onShowDrawer ) {
  if (!window.labels) {
    window.labels = {
      "les_personnes_a_assurer" : "Les personnes à assurer",
      "information_de_l_assurer" : "Information de l'assurer n ",
      "ajouter_un_assurer" : "Ajouter un Assuré",
      "ajouter" : "Ajouter",
      "modifier" : "Modifier",
      "annuler" : "Annuler",
      "retour" : "Retour",
      "suivant" : "Suivant",
      "nom" : "Nom",
      "prenom" : "Prénom",
      "date_de_naissance" : "Date de naissance",
      "num_passeport" : "Numéro passeport",
      "nationalite" : "Nationalité",
      "requis" : "Réquis",
      "conjoint": {
        "titre": "Conjoint",
        "conjoint-nom": "Nom",
        "conjoint-prenom": "Prénom",
        "conjoint-datenais": "Date de naissance",
        "conjoint-passeport":"Passeport",
        "conjoint-nationalite":"Nationalité"
      },
      "enfants": {
        "titre": "Enfant (moins de 21 ans)",
        "enfant-num": "N°*num*",
        "enfant-nom": "Nom",
        "enfant-prenom": "Prénom",
        "enfant-datenaiss": "Date de naissance",
        "enfant-passeport":"Passeport",
        "enfant-nationalite":"Nationalité",
        "enfant-action": ""
      },
      "buttons": {
        "ajout": "Ajouter",
        "suppression": "Supprimer"
      },
      'accordion': {
        "activeText": "Membres de la famille",
        "inactiveText": "Gérer les membres de la famille"
      }
    }
  }

  /**
   * Cette fonction sert à créer un widget de saisit des membres
   * de la famille de l'assuré principal
   *
   * @param {*} personne
   * @param {*} indice
   */
  window.buildUiFamilleInListAssurer = function (personne, indice, onChangeInFamille) {
    const personneIndice = indice;
    const parent = document.querySelector(".widgetFamille");
    let widget = {};
    widget = document.createElement("div");
    widget.className = "widget-famille";
    parent.appendChild(widget);

    ReactDOM.render(
        <ConfigProvider locale={frLocale}>
          <Personne personne={personne} indice={personneIndice} onChangeInFamille={onChangeInFamille} />
        </ConfigProvider>,
        widget
    );
  };

  var selector = document.getElementById('root');
  if ( selector!== null ) {
    ReactDOM.unmountComponentAtNode(selector);
    ReactDOM.render(
        <React.StrictMode>
          <App data={data} famille={famille} onChange={onChange} onShowDrawer={onShowDrawer}/>
        </React.StrictMode>,
        selector
    );
  }
}



// If you want to start measuring performance in your app, pass a function
// to log results (for example: reportWebVitals(console.log))
// or send to an analytics endpoint. Learn more: https://bit.ly/CRA-vitals
