import React, { Component } from "react";
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faTrash } from '@fortawesome/free-solid-svg-icons';
import { Input, DatePicker, Row, Col, Tooltip, Typography } from 'antd';
import moment from 'moment';
import 'moment/locale/fr';
import { disabledFutureDate, FORMAT_FR } from '../../utils/dateUtil';
import localeFr from 'antd/es/date-picker/locale/fr_FR';
import styled from "styled-components";
const {Title} = Typography;
moment().locale("fr");
const RowStyled = styled(Row)`
          margin-top: 5px;
          margin-bottom: 5px;
        `;
class Enfant extends Component {
    constructor(props) {
        super(props);
        this.updateNom = this.updateNom.bind(this);
        this.updatePrenom = this.updatePrenom.bind(this);
        this.updateDateNaiss = this.updateDateNaiss.bind(this);
        this.onEnfantChange = this.onEnfantChange.bind(this);
        this.onDeleteEnfant = this.onDeleteEnfant.bind(this);
        this.updatePasseport = this.updatePasseport.bind(this);
        this.updateNationalite = this.updateNationalite.bind(this);
    }

    componentDidMount() {
    }

    render() {
        const { indice, layout, required } = this.props;
        const {
            "nom": enom,
            "prenom": eprenom,
            "dateNaiss": edateNaiss,
            "passeport": epasseport,
            "nationalite": enationalite
        } = this.props.enfant;
        const labels = window.labels;
        const edateNaissObject = edateNaiss ? moment(edateNaiss, "DD/MM/YYYY") : "";

        if (layout && layout.toLowerCase() === "desktop")
            return (
                <>
                    <Title level={3}>{labels.enfants.titre ? labels.enfants.titre : "Enfants"}</Title>
                    <RowStyled  gutter={[8, 16]}>
                        <Col xs={12} sm={8}>
                            <Tooltip title={labels.enfants["enfant-nom"] ? labels.enfants["enfant-nom"] : "Nom"}>
                                <Input
                                    onChange={this.updateNom}
                                    value={enom || ""}
                                    required={ required }
                                    aria-required={ required }
                                    placeholder={labels.enfants["enfant-nom"] ? labels.enfants["enfant-nom"] : "Nom"} />
                            </Tooltip>
                        </Col>
                        <Col xs={12} sm={8}>
                            <Tooltip title={labels.enfants["enfant-prenom"] ? labels.enfants["enfant-prenom"] : "Prenom"}>
                                <Input
                                    onChange={this.updatePrenom}
                                    value={eprenom || ""}
                                    required={ required }
                                    aria-required={ required }
                                    placeholder={labels.enfants["enfant-prenom"] ? labels.enfants["enfant-prenom"] : "Prenom"}
                                />
                            </Tooltip>
                        </Col>
                        <Col xs={12} sm={8}>
                            <Tooltip title={labels.enfants["enfant-datenaiss"] ? labels.enfants["enfant-datenaiss"] : "Date de naissance"}>
                                <DatePicker
                                    disabledDate={disabledFutureDate}
                                    format={FORMAT_FR}
                                    locale={localeFr}
                                    required={ required }
                                    aria-required={ required }
                                    placeholder={labels.enfants["enfant-datenaiss"] ? labels.enfants["enfant-datenaiss"] : "Date de naissance"}
                                    onChange={this.updateDateNaiss} value={edateNaissObject} />
                            </Tooltip>
                        </Col>
                    </RowStyled>
                    <RowStyled gutter={[8, 16]}>
                        <Col xs={12} sm={8}>
                            <Tooltip title={labels.enfants["enfant-passeport"] ? labels.enfants["enfant-passeport"] : "Passeport"}>
                                <Input
                                    onChange={this.updatePasseport}
                                    value={epasseport || ""}
                                    required={ required }
                                    aria-required={ required }
                                    placeholder={labels.enfants["enfant-passeport"] ? labels.enfants["enfant-passeport"] : "Passeport"} />
                            </Tooltip>
                        </Col>
                        <Col xs={12} sm={8}>
                            <Tooltip title={labels.enfants["enfant-nationalite"] ? labels.enfants["enfant-nationalite"] : "Nationalité"}>
                                <Input
                                    onChange={this.updateNationalite}
                                    value={enationalite || ""}
                                    required={ required }
                                    aria-required={ required }
                                    placeholder={labels.enfants["enfant-nationalite"] ? labels.enfants["enfant-nationalite"] : "Nationalité"} />
                            </Tooltip>
                        </Col>

                        <Col xs={12} sm={8}>
                            <button
                                type="button"
                                className="btn btn-error button-secondary"
                                style={{ padding: '0 1rem' }}
                                onClick={this.onDeleteEnfant}
                                title={labels.buttons.suppression}>
                                <FontAwesomeIcon icon={faTrash} />
                            </button>
                        </Col>
                    </RowStyled>
                </>
            );

        else if (layout && layout.toLowerCase() === "mobile") {
            return (
                <RowStyled className='enfant' gutter={[8, 16]}>
                    <Col span={24}>
                        {labels.enfants["enfant-num"] ? labels.enfants["enfant-num"].replace("*num*", indice + 1) : "Enfant n°" + (indice + 1)}
                    </Col>
                    <Col span={12}>
                        <Tooltip title={labels.enfants["enfant-nom"] ? labels.enfants["enfant-nom"] : "Nom"}>
                            <Input
                                onChange={this.updateNom}
                                value={enom || ""}
                                placeholder={labels.enfants["enfant-nom"] ? labels.enfants["enfant-nom"] : "Nom"}
                            />
                        </Tooltip>
                    </Col>
                    <Col span={12}>
                        <Tooltip title={labels.enfants["enfant-prenom"] ? labels.enfants["enfant-prenom"] : "Prenom"}>
                            <Input
                                onChange={this.updatePrenom}
                                value={eprenom || ""}
                                placeholder={labels.enfants["enfant-prenom"] ? labels.enfants["enfant-prenom"] : "Prenom"}
                            />
                        </Tooltip>
                    </Col>
                    <Col span={12}>
                        <Tooltip title={labels.enfants["enfant-passeport"] ? labels.enfants["enfant-passeport"] : "Passeport"}>
                            <Input
                                onChange={this.updatePasseport}
                                value={epasseport || ""}
                                required={ required }
                                aria-required={ required }
                                placeholder={labels.enfants["enfant-passeport"] ? labels.enfants["enfant-passeport"] : "Passeport"} />
                        </Tooltip>
                    </Col>
                    <Col span={12}>
                        <Tooltip title={labels.enfants["enfant-nationalite"] ? labels.enfants["enfant-nationalite"] : "Nationalité"}>
                            <Input
                                onChange={this.updateNationalite}
                                value={enationalite || ""}
                                required={ required }
                                aria-required={ required }
                                placeholder={labels.enfants["enfant-nationalite"] ? labels.enfants["enfant-nationalite"] : "Nationalité"} />
                        </Tooltip>
                    </Col>
                    <Col>
                        <Tooltip title={labels.enfants["enfant-datenaiss"] ? labels.enfants["enfant-datenaiss"] : "Date de naissance"}>
                            <DatePicker
                                disabledDate={disabledFutureDate}
                                format={FORMAT_FR}
                                locale={localeFr}
                                showToday={false}
                                inputReadOnly={false}
                                placeholder={labels.enfants["enfant-datenaiss"] ? labels.enfants["enfant-datenaiss"] : "Sa date de naissance"}
                                onChange={this.updateDateNaiss} value={edateNaissObject} />
                        </Tooltip>
                    </Col>
                    <Col>
                        <button type="button" className="btn btn-error button-secondary" onClick={this.onDeleteEnfant} title={labels.buttons.suppression}>
                            <FontAwesomeIcon icon={faTrash} />
                        </button>
                    </Col>
                </RowStyled>
            );
        }
    }
    onEnfantChange(enfant, indice) {
        this.props.onEnfantChange(enfant, indice);
    }
    updateNom(e) {
        const { enfant } = this.props;
        enfant["nom"] = e.target.value;
        this.onEnfantChange(enfant, this.props.indice);
    }
    updatePrenom(e) {
        const { enfant } = this.props;
        enfant["prenom"] = e.target.value;
        this.onEnfantChange(enfant, this.props.indice);
    }
    updateDateNaiss(date, dateString) {
        const { enfant } = this.props;
        enfant["dateNaiss"] = dateString;
        this.onEnfantChange(enfant, this.props.indice);
    }
    onDeleteEnfant(e) {
        this.props.onDeleteEnfant(this.props.indice);
    }
    updatePasseport(e) {
        const { enfant } = this.props;
        enfant["passeport"] = e.target.value;
        this.onEnfantChange(enfant, this.props.indice);
    }
    updateNationalite(e) {
        const { enfant } = this.props;
        enfant["nationalite"] = e.target.value;
        this.onEnfantChange(enfant, this.props.indice);
    }
}
export default Enfant;