import React, { Component } from "react";
import "./personne.css";
import Enfant from "./enfant/enfant";
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faPlus } from '@fortawesome/free-solid-svg-icons';
import {Collapse, Typography, Row, Col, notification, Tooltip, Input, DatePicker} from 'antd';
import moment from 'moment';
import 'moment/locale/fr';
import { FORMAT_FR } from '../utils/dateUtil';
import styled from "styled-components";
import localeFr from "antd/es/date-picker/locale/fr_FR";
const { Panel } = Collapse;
const { Title } = Typography;
moment().locale("fr");

const RowStyled = styled(Row)`
          margin-top: 5px;
          margin-bottom: 5px;
        `;
class Personne extends Component {
    state = {
        personne: {
            /*
            "personne--conjoint-nom": "",
            "personne--conjoint-prenom": "",
            "personne--conjoint-datenais": "",
            "personne--enfants": ""
            */
        },
        indice: 0,
        maxEnfant: 5,
        headerText: "",
        infoConjointRequired: true,
        infoEnfantsRequired: true,
    }
    constructor(props) {
        super(props);

        this.state = {
            personne: props.personne,
            indice: props.indice,
            maxEnfant: 5,
            infoConjointRequired: true,
            infoEnfantsRequired: true,
        };
        this.updateNom = this.updateNom.bind(this);
        this.updatePrenom = this.updatePrenom.bind(this);
        this.updateDateNaiss = this.updateDateNaiss.bind(this);
        this.updateConjointPasseport = this.updateConjointPasseport.bind(this);
        this.updateConjointNationalite = this.updateConjointNationalite.bind(this);
        this.onEnfantChange = this.onEnfantChange.bind(this);
        this.createEnfant = this.createEnfant.bind(this);
        this.createConjoint = this.createConjoint.bind(this);
        this.onDeleteEnfant = this.onDeleteEnfant.bind(this);
        this.onChangeInFamille = this.onChangeInFamille.bind(this);
    }
    componentDidMount() {
        const { personne, indice } = this.props;
        const labels = window.labels;
        const infoConjointRequired = (typeof window.infoConjointRequired !== 'undefined') ? window.infoConjointRequired : true;
        const infoEnfantsRequired = (typeof window.infoEnfantsRequired !== 'undefined') ? window.infoEnfantsRequired : true;
        this.setState({
            personne: personne,
            indice: indice,
            headerText: labels.accordion.inactiveText,
            maxEnfant: labels.maxEnfant ? labels.maxEnfant : 5,
            infoConjointRequired: infoConjointRequired,
            infoEnfantsRequired: infoEnfantsRequired
        });
        const jsonEnfants = personne["personne--enfants"] || "[]";
        // if (JSON.parse(jsonEnfants).length === 0) this.createEnfant();
    }

    render() {
        const labels = window.labels;
        const {infoConjointRequired, infoEnfantsRequired} = this.state;
        let arrayEnfants = [];

        const {
            "personne--conjoint-nom": cnom,
            "personne--conjoint-prenom": cprenom,
            "personne--conjoint-datenais": cdateNaiss,
            "personne--conjoint-passeport": cpasseport,
            "personne--conjoint-nationalite": cnationalite,
            "personne--enfants": penfants,
        } = this.state.personne;

        let arrayConjoint = [this.state.personne];

        const { indice, headerText } = this.state;

        if (penfants) {
            arrayEnfants = penfants !== "" ? JSON.parse(penfants) : [];
        }

        const ListEnfantsDesktop = arrayEnfants.map((enfant, key) => {
            return (
                <Enfant
                    key={key}
                    indice={key}
                    enfant={enfant}
                    onEnfantChange={this.onEnfantChange}
                    onDeleteEnfant={this.onDeleteEnfant}
                    layout="desktop" //desktop
                    required={infoEnfantsRequired}
                />
            );
        });

        const ListEnfantsMobile = arrayEnfants.map((enfant, key) => {
            return (
                <Enfant
                    key={key}
                    indice={key}
                    enfant={enfant}
                    onEnfantChange={this.onEnfantChange}
                    onDeleteEnfant={this.onDeleteEnfant}
                    layout="mobile" //mobile
                    required={infoEnfantsRequired}
                />
            );
        });
        const cdateNaissObject = cdateNaiss ? moment(cdateNaiss, "DD/MM/YYYY") : "";

        return (
            <Collapse defaultActiveKey={['0']} onChange={this.changeHeaderText.bind(this)} >
                <Panel header={headerText} key='0'>
                    <Title level={3}>{labels.conjoint.titre ? labels.conjoint.titre : "Conjoint"}</Title>
                    <RowStyled gutter={[8, 16]}>
                        <Col xs={12} sm={8}>
                            <Tooltip title={labels.conjoint["conjoint-nom"] ? labels.conjoint["conjoint-nom"] : "Nom"}>
                                <Input name={"personne[" + indice + "][personne--conjoint-nom]"}
                                    placeholder={labels.conjoint["conjoint-nom"] ? labels.conjoint["conjoint-nom"] : "Nom"}
                                    required={infoConjointRequired}
                                    aria-required={infoConjointRequired}
                                    onChange={this.updateNom} value={cnom || ""} /></Tooltip>
                        </Col>
                        <Col xs={12} sm={8}>
                            <Tooltip title={labels.conjoint["conjoint-prenom"] ? labels.conjoint["conjoint-prenom"] : "Prenom"}>
                                <Input name={"personne[" + indice + "][personne--conjoint-prenom]"}
                                    placeholder={labels.conjoint["conjoint-prenom"] ? labels.conjoint["conjoint-prenom"] : "Prenom"}
                                    required={infoConjointRequired}
                                    aria-required={infoConjointRequired}
                                    onChange={this.updatePrenom} value={cprenom || ""} />
                            </Tooltip>
                        </Col>
                        <Col xs={24} sm={8}>
                            <Tooltip title={labels.conjoint["conjoint-datenais"] ? labels.conjoint["conjoint-datenais"] : "Date de naissance"}>
                                <DatePicker name={"personne[" + indice + "][personne--conjoint-datenais]"}
                                    onChange={this.updateDateNaiss}
                                    value={cdateNaissObject || ""}
                                    locale={localeFr}
                                    format={FORMAT_FR}
                                    disabledDate={c => c && c > moment()}
                                    inputReadOnly={false}
                                    showToday={false}
                                    required={infoConjointRequired}
                                    aria-required={infoConjointRequired}
                                    placeholder={labels.conjoint["conjoint-datenais"] ? labels.conjoint["conjoint-datenais"] : "Date de naissance"}
                                />
                            </Tooltip>
                        </Col>
                    </RowStyled>
                    <RowStyled gutter={[8, 16]}>
                        <Col xs={12} sm={12}>
                            <Tooltip title={labels.conjoint["conjoint-passeport"] ? labels.conjoint["conjoint-passeport"] : "Passeport"}>
                                <Input name={"personne[" + indice + "][personne--conjoint-passeport]"}
                                    placeholder={labels.conjoint["conjoint-passeport"] ? labels.conjoint["conjoint-passeport"] : "Passeport"}
                                    required={infoConjointRequired}
                                    aria-required={infoConjointRequired}
                                    onChange={this.updateConjointPasseport} value={cpasseport || ""} />
                            </Tooltip>
                        </Col>
                        <Col xs={12} sm={12}>
                            <Tooltip title={labels.conjoint["conjoint-nationalite"] ? labels.conjoint["conjoint-nationalite"] : "Nationalité"}>
                                <Input name={"personne[" + indice + "][personne--conjoint-nationalite]"}
                                    placeholder={labels.conjoint["conjoint-nationalite"] ? labels.conjoint["conjoint-nationalite"] : "Nationalité"}
                                    required={infoConjointRequired}
                                    aria-required={infoConjointRequired}
                                    onChange={this.updateConjointNationalite} value={cnationalite || ""} />
                            </Tooltip>
                        </Col>
                    </RowStyled>

                    {typeof this.props.options != 'undefined' && this.props.options.widget === 'diaspora' && (
                        <button type="button"
                                className="btn btn-success button-primary"
                                title={labels.buttons.ajout ? labels.buttons.ajout : "Ajouter"}
                                onClick={this.createConjoint}>
                            <FontAwesomeIcon icon={faPlus} />
                        </button>
                    )}
                    {ListEnfantsDesktop.length === 0 && 
                        <Title level={3}>{labels.enfants.titre ? labels.enfants.titre : "Enfants"}</Title>
                    }
                    {ListEnfantsDesktop.length > 0 &&
                        <RowStyled>
                            <Col span={24}>
                                <Row>
                                    <Col xs={24} sm={0}>
                                        {ListEnfantsMobile}
                                    </Col>
                                </Row>
                                <Row>
                                    <Col xs={0} sm={24}>
                                        {ListEnfantsDesktop}
                                    </Col>
                                </Row>
                            </Col>
                        </RowStyled>
                    }
                    <button type="button" className="btn btn-success button-primary" title={labels.buttons.ajout ? labels.buttons.ajout : "Ajouter"} onClick={this.createEnfant}>
                        <FontAwesomeIcon icon={faPlus} />
                    </button>
                    <input type='hidden' name={"personne[" + indice + "][personne--enfants]"} value={penfants ? penfants : ""} onChange={() => { }} />
                </Panel>
            </Collapse>
        );
    }
    changeHeaderText(keys) {
        const labels = window.labels;
        if (keys.length > 0) {
            this.setState({
                headerText: labels.accordion.activeText
            })
        } else {
            this.setState({
                headerText: labels.accordion.inactiveText
            })
        }
    }
    updateNom(e) {
        const { personne } = this.state;
        personne["personne--conjoint-nom"] = e.target.value;
        this.setState({ personne: personne });
        this.onChangeInFamille( this.state.personne, this.state.indice);
    }
    updatePrenom(e) {
        const { personne } = this.state;
        personne["personne--conjoint-prenom"] = e.target.value;
        this.setState({ personne: personne });
        this.onChangeInFamille( this.state.personne, this.state.indice);
    }
    updateDateNaiss(date, dateString) {
        const { personne } = this.state;
        personne["personne--conjoint-datenais"] = dateString;
        this.setState({ personne: personne });
        this.onChangeInFamille( this.state.personne, this.state.indice);
    }
    onEnfantChange(enfant, indice) {
        const { personne } = this.state;
        const enfantsArray = JSON.parse(personne["personne--enfants"]);
        enfantsArray[indice] = enfant;
        personne["personne--enfants"] = JSON.stringify(enfantsArray);
        this.setState({
            personne: personne
        });
        this.onChangeInFamille( this.state.personne, this.state.indice);
    }
    createConjoint() {

    }
    createEnfant() {
        const { personne, maxEnfant } = this.state;
        const labels = window.labels;
        const enfantsArray = JSON.parse(personne["personne--enfants"] ? personne["personne--enfants"] : "[]");
        if (enfantsArray.length < maxEnfant) {
            enfantsArray.push({
                "nom": "",
                "prenom": "",
                "dateNaiss": ""
            });
            personne["personne--enfants"] = JSON.stringify(enfantsArray);
            this.setState({
                personne: personne
            });
            this.onChangeInFamille( this.state.personne, this.state.indice);
        } else {
            try {
                notification.warning({
                    description: labels.errors.maxEnfant.replace("*number*", maxEnfant)
                });
            } catch (e) {
                notification.warning({
                    description: `Nombre d'enfants limité à ${maxEnfant}, pour plus de renseignements contactez notre équipe pour plus d'informations`
                });
            }
        }
    }
    onDeleteEnfant(indice) {
        const { personne } = this.state;
        const enfantsArray = JSON.parse(personne["personne--enfants"]);
        const newArray = enfantsArray.filter((e, i) => {
            return i !== indice;
        });
        personne["personne--enfants"] = JSON.stringify(newArray);
        this.setState({
            personne: personne
        });
        this.onChangeInFamille( this.state.personne, this.state.indice);
    }
    updateConjointPasseport (e) {
        const { personne } = this.state;
        personne['personne--conjoint-passeport'] = e.target.value;
        this.setState({ personne: personne});
        this.onChangeInFamille( this.state.personne, this.state.indice);
    }
    updateConjointNationalite (e) {
        const { personne } = this.state;
        personne['personne--conjoint-nationalite'] = e.target.value;
        this.setState({ personne: personne});
        this.onChangeInFamille( this.state.personne, this.state.indice);
    }
    onChangeInFamille ( personne, indice ) {
        this.props.onChangeInFamille( personne, indice );
    }
}
export default Personne;