import React from 'react';
import {Table, TableBody, TableHead, TableRow, TableCell, Button, Link} from '@material-ui/core';
import {labels} from '../config/label';
import {CreditCardOutlined } from '@material-ui/icons';

const getContratListe = async () => {
    const response = await fetch(document.location.pathname + '/ajax/contrat-liste');
    const json = await response.json();
    return json;
};
const TableContrat = function ({withCommission=true}) {
    const [isLoading, setLoading] = React.useState(false);
    const [data, setData] = React.useState([]);
    React.useEffect(() => {
        async function autoLoadContrat() {
            setLoading(true);
            const json = await getContratListe();
            setData(json);
            setLoading(false);
        }

        autoLoadContrat();
    }, [setLoading, setData, getContratListe]);
    const listeContrats = data.map((resource, i) => {
        return (
            <TableRow key={i}>
                <TableCell>
                    <Link href={"?id-contrat=" + resource.idContrat + "&tab=contrats"}>
                        <svg aria-hidden="true" focusable="false" data-prefix="far" data-icon="eye" role="img"
                             xmlns="http://www.w3.org/2000/svg" viewBox="0 0 576 512"
                             className="eye">
                            <path fill="currentColor"
                                  d="M288 144a110.94 110.94 0 0 0-31.24 5 55.4 55.4 0 0 1 7.24 27 56 56 0 0 1-56 56 55.4 55.4 0 0 1-27-7.24A111.71 111.71 0 1 0 288 144zm284.52 97.4C518.29 135.59 410.93 64 288 64S57.68 135.64 3.48 241.41a32.35 32.35 0 0 0 0 29.19C57.71 376.41 165.07 448 288 448s230.32-71.64 284.52-177.41a32.35 32.35 0 0 0 0-29.19zM288 400c-98.65 0-189.09-55-237.93-144C98.91 167 189.34 112 288 112s189.09 55 237.93 144C477.1 345 386.66 400 288 400z"
                                  className=""></path>
                        </svg>
                        {resource.num_contrat}
                    </Link>
                </TableCell>
                <TableCell>
                    {resource.nom}
                </TableCell>
                <TableCell>
                    {resource.prenoms}
                </TableCell>
                <TableCell>
                    <svg width={20} height={20}>
                        <image xlinkHref={resource.provenance_pays.flag} width={20} height={20}/>
                    </svg>
                    <p> {resource.provenance_pays.name}</p>
                </TableCell>
                <TableCell>
                    <svg width={20} height={20}>
                        <image  xlinkHref={resource.destination_pays.flag}  width={20} height={20}/>
                    </svg>
                    <p> {resource.destination_pays.name}</p>
                </TableCell>
                <TableCell>
                    {resource.date_depart}
                </TableCell>
                <TableCell>
                    {resource.date_fin}
                </TableCell>
                <TableCell>
                    {resource.montant}
                </TableCell>
                {withCommission &&
                <TableCell>
                    {resource.commission}
                </TableCell>
                }
                <TableCell>
                    <Button className={"btn btn-xs btn-success declarer-sinistre"} data-content={resource.idContrat}
                            data-toggle={"modal"} data-target={"#formulaire_sinistre"}><CreditCardOutlined/> {labels.LABEL_DECLARER_SINISTRE}</Button>
                </TableCell>
            </TableRow>
        );
    })
    return (
        <div className={"tableContrat"}>
            <Table>
                <TableHead>
                    <TableRow>
                        <TableCell component={'th'}>
                            {labels.LABEL_NUM_CONTRAT}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_NOM}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_PRENOMS}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_PROVENANCE_PAYS}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_DESTINATION_PAYS}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_DEBUT}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_FIN}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_MONTANT}
                        </TableCell>
                        {withCommission &&
                        <TableCell component={'th'}>
                            {labels.LABEL_COMMISSION}
                        </TableCell>
                        }
                        <TableCell component={'th'}>
                            {labels.LABEL_ACTIONS}
                        </TableCell>
                    </TableRow>
                </TableHead>
                <TableBody>
                    {isLoading &&
                    <TableRow>
                        <TableCell colSpan="10">
                            <p>{labels.LABEL_LOADING}</p>
                        </TableCell>
                    </TableRow>
                    }
                    {!isLoading && listeContrats.length === 0 &&
                    <TableRow>
                        <TableCell colSpan="10">
                            <div className="alert alert-info">{labels.LABEL_EMPTY}</div>
                        </TableCell>
                    </TableRow>
                    }
                    {!isLoading && listeContrats}
                </TableBody>
            </Table>
        </div>
    )
}
export default TableContrat;