import React from 'react';
import {Button, IconButton, Link, Table, TableBody, TableCell, TableHead, TableRow} from '@material-ui/core';
import {labels} from '../config/label';
import {CreditCardOutlined, DeleteOutlined} from '@material-ui/icons';
import makeStyles from "@material-ui/core/styles/makeStyles";

const useStyles = makeStyles({
    root: {
        maxWidth: "100%",
        margin: '2px',
        borderRadius: 20,
        "& .MuiCardHeader-root": {
            padding: 10,
            textAlign: "center",
            color: "#339999",
        }
    },
    cardFooter: {
        padding: "1rem",
        "& *": {
            color: "#339999"
        }
    },
    btnOrange: {
        backgroundColor: "#FF6600",
        borderRadius: "10px 0px 0px 10px",
        boxShadow: "0px 3px 1px -2px rgba(0,0,0,0.2), 0px 2px 2px 0px rgba(0,0,0,0.14), 0px 1px 5px 0px rgba(0,0,0,0.12)",
        color: "white",
        "&:hover": {
            backgroundColor: "#e85f00",
            boxShadow: "0px 2px 4px -1px rgba(0,0,0,0.2), 0px 4px 5px 0px rgba(0,0,0,0.14), 0px 1px 10px 0px rgba(0,0,0,0.12)"
        }
    },
    rightBorderRadius: {
        borderRadius: "0px 10px 10px 0px",
    },
    hoverBlack: {
        "&:hover *": {
            color: "black"
        }
    },
    buttonHeigth: {
        padding: "9px"
    }
});

const getDevisListe = async () => {
    const response = await fetch(document.location.pathname + '/ajax/devis-liste');
    const json = await response.json();
    return json;
};

const TableDevis = function ({afficherPayerEspece = false, withCommission = true}) {
    const classes = useStyles();
    const [isLoading, setLoading] = React.useState(false);
    const [data, setData] = React.useState([]);
    React.useEffect(() => {
        async function autoLoadDevis() {
            setLoading(true);
            const json = await getDevisListe();
            setData(json);
            setLoading(false);
        }

        autoLoadDevis();
    }, [setLoading, setData, getDevisListe]);
    const listeDevis = data.map((resource, i) => {
        return (
            <TableRow key={i}>
                <TableCell>
                    <Link href={"?id-devis=" + resource.idContrat + "&tab=devis"}>
                        <svg aria-hidden="true" focusable="false" data-prefix="far" data-icon="eye" role="img"
                             xmlns="http://www.w3.org/2000/svg" viewBox="0 0 576 512"
                             className="eye">
                            <path fill="currentColor"
                                  d="M288 144a110.94 110.94 0 0 0-31.24 5 55.4 55.4 0 0 1 7.24 27 56 56 0 0 1-56 56 55.4 55.4 0 0 1-27-7.24A111.71 111.71 0 1 0 288 144zm284.52 97.4C518.29 135.59 410.93 64 288 64S57.68 135.64 3.48 241.41a32.35 32.35 0 0 0 0 29.19C57.71 376.41 165.07 448 288 448s230.32-71.64 284.52-177.41a32.35 32.35 0 0 0 0-29.19zM288 400c-98.65 0-189.09-55-237.93-144C98.91 167 189.34 112 288 112s189.09 55 237.93 144C477.1 345 386.66 400 288 400z"
                                  className=""></path>
                        </svg>
                        {resource.num_contrat}
                    </Link>
                </TableCell>
                <TableCell>
                    {resource.nom}
                </TableCell>
                <TableCell>
                    {resource.prenoms}
                </TableCell>
                <TableCell>
                    <svg width={20} height={20}>
                        <image xlinkHref={resource.provenance_pays.flag} width={20} height={20}/>
                    </svg>
                    <p> {resource.provenance_pays.name}</p>
                </TableCell>
                <TableCell>
                    <svg width={20} height={20}>
                        <image xlinkHref={resource.destination_pays.flag} width={20} height={20}/>
                    </svg>
                    <p> {resource.destination_pays.name}</p>
                </TableCell>
                <TableCell>
                    {resource.date_depart}
                </TableCell>
                <TableCell>
                    {resource.date_fin}
                </TableCell>
                <TableCell>
                    {resource.montant}
                </TableCell>
                {withCommission &&
                <TableCell>
                    {resource.commission}
                </TableCell>
                }
                <TableCell>
                    <div className={"btn-group btn-in-sm"} role={"group"}>
                        {!resource.mise_a_jour && (
                            <>
                                <Button type={"submit"} variant={"contained"} color={"primary"}
                                        className={classes.btnOrange}
                                        form={resource.dalenysFormId}><CreditCardOutlined/>{resource.buttonText}
                                </Button>
                                <div dangerouslySetInnerHTML={{__html: resource.paymentButton}}></div>
                                {afficherPayerEspece &&
                                <Link href={document.location.pathname + "/bypasspaiement/" + resource.idContrat}
                                      className={"btn btn-xs text-white btn-success btn-payerES show-loader"}>{resource.buttonTextES}</Link>
                                }
                            </>
                        )}

                        {resource.mise_a_jour &&
                            <Link href={ '?id-devis='+ resource.idContrat +'&tab=devis&edit=true'} className={ classes.btnOrange + " btn text-white " + classes.buttonHeigth }>{ resource.buttonTextMAJ }</Link>
                        }
                        <IconButton color={"secondary"} data-id={resource.idContrat}
                                    className={classes.rightBorderRadius + " " + classes.hoverBlack + " MuiButton-contained btn btn-secondary delete-devis"}><DeleteOutlined/></IconButton>
                    </div>
                </TableCell>
            </TableRow>
        );
    });
    return (
        <div className={"tableDevis"}>
            <Table>
                <TableHead>
                    <TableRow>
                        <TableCell component={'th'}>
                            {labels.LABEL_NUM_DEVIS}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_NOM}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_PRENOMS}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_PROVENANCE_PAYS}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_DESTINATION_PAYS}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_DEBUT}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_FIN}
                        </TableCell>
                        <TableCell component={'th'}>
                            {labels.LABEL_MONTANT}
                        </TableCell>
                        {withCommission &&
                        <TableCell component={'th'}>
                            {labels.LABEL_COMMISSION}
                        </TableCell>
                        }
                        <TableCell component={'th'}>
                            {labels.LABEL_ACTIONS}
                        </TableCell>
                    </TableRow>
                </TableHead>
                <TableBody>
                    {isLoading &&
                    <TableRow>
                        <TableCell colSpan="10">
                            <p>{labels.LABEL_LOADING}</p>
                        </TableCell>
                    </TableRow>
                    }

                    {!isLoading && listeDevis.length === 0 &&
                    <TableRow>
                        <TableCell colSpan="10">
                            <div className="alert alert-info">{labels.LABEL_EMPTY}</div>
                        </TableCell>
                    </TableRow>
                    }
                    {!isLoading && listeDevis}
                </TableBody>
            </Table>
        </div>
    )
};

export default TableDevis;