import React from 'react';
import makeStyles from "@material-ui/core/styles/makeStyles";
import LangContext from "../../context/lang.context";
import {Hidden, MenuItem, Select, Tooltip} from "@material-ui/core";
import * as exports from "react-flags-select";
Object.entries(exports).forEach(([name, exported]) => global[name.toUpperCase()] = exported);
const Flag = ({flagCode}) => {
    if (flagCode.length > 2) {
        return <img style={{ width: "30px", height: "auto"}} src={flagCode} />
    } else {
        return global[flagCode](); // vu qu'un composant est une fonction à la base. il faut l'appeler pour faire le rendu
    }
}

const useStyles = makeStyles(theme => ({
    block: {
        display: "block !important",
        textAlign: "center",
        paddingLeft: "1rem !important"
    },
    liste: {
        backgroundColor: "white",
        borderRadius: "8px",
        width: "fit-content !important",
        height: "auto !important",
        padding: "0 !important"
    },
    listeItem: {
        backgroundColor: "transparent !important",
        border: "none !important",
        boxShadow: "none !important",
        borderRadius: "0 !important",
        "&:first-child": {
            borderRadius: "8px 0 0 8px !important",
        },
        "&:last-child": {
            paddingRight: 5,
            borderRadius: "0 8px 8px 0 !important",

        },
        "&:hover , &.active": {
            backgroundColor: "#339999 !important"
        },
        [theme.breakpoints.down('md')]: {
            width: "25%"
        }
    }
}));
const getCookie = (cname) => {
    var name = cname + "=";
    var ca = document.cookie.split(';');
    for (var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) == ' ') {
            c = c.substring(1);
        }
        if (c.indexOf(name) == 0) {
            return c.substring(name.length, c.length);
        }
    }
    return "";
}
const setCookie = (cname, cvalue, exdays) => {
    var d = new Date();
    d.setTime(d.getTime() + (exdays * 24 * 60 * 60 * 1000));
    var expires = "expires=" + d.toUTCString();
    document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/";
};
const LangSwitcher = () => {
    const classes = useStyles();
    const ctx = React.useContext(LangContext);
    const [langs, setLangs] = React.useState(() => []);
    const [currentLang, setCurrentLang] = React.useState("");
    const cookieName = 'mc_locale';
    React.useEffect(() => {
        async function fetchLangs () {
            const json = await ctx.langs();
            setLangs(json);
        }
        fetchLangs();
    }, [setLangs]);

    React.useEffect(() => {
        const lang = getCookie(cookieName);
        if (lang && lang !== "") setCurrentLang(lang);
    }, [setCurrentLang]);

    const changeLang = (e, lang) => {
        setCookie(cookieName, lang.lang, 365);
        setCurrentLang(lang.lang);
        window.location.reload(true);
        e.preventDefault();
    };
    const selectLang = (e, lang) => {
        setCookie(cookieName, lang, 365);
        setCurrentLang(lang);
        window.location.reload(true);
        e.preventDefault();
    }
    var LangsList = [];
    var LangsOptions = [];
    if(langs){
        LangsList = langs.map((lang, i) => {
            return <li
                className={(currentLang === lang.lang ? "active " : "") + classes.listeItem + " justify-content-start"}
                key={i}>
                <Tooltip title={lang.nativeLang}>
                    <a className={classes.block} href={"#"} onClick={ev => changeLang(ev, lang)}>
                    <span style={{
                        fontSize: "2.5em"
                    }}>
                        <Flag flagCode={lang.flag}/>
                    </span>
                    </a>
                </Tooltip>
            </li>
        });
        LangsOptions = langs.map((lang, i) => {
            return <MenuItem value={lang.lang} key={i}>
                <span style={{
                    fontSize: "2.5em"
                }}>
                    <Flag flagCode={lang.flag}/>
                </span>&nbsp;
                {lang.nativeLang}</MenuItem>
        });
    }
    return (
        <>
            <Hidden smDown>
                <div className={"d-flex justify-content-start align-items-center mx-4"}>
                    <ul className={classes.liste + " d-flex flex-wrap justify-content-center"}>
                        {LangsList}
                    </ul>
                </div>
            </Hidden>
            <Hidden mdUp>
                <div className={"d-flex justify-content-start align-items-center ml-4"}>
                    <Select defaultValue={currentLang} onChange={e => selectLang(e, e.target.value)}
                            style={{'color': 'white'}}>
                        <MenuItem value="">
                            <em>Choisir</em>
                        </MenuItem>
                        {LangsOptions}
                    </Select>
                </div>
            </Hidden>
        </>
    )
}
export default LangSwitcher;