import React, {useState} from "react"
import ReactDom from "react-dom"
import {DatePicker, Row, Col, Space, ConfigProvider, Input, Button, Select} from "antd"

import moment from "moment";

import('moment/locale/fr');
import fr_FR from 'antd/lib/locale/fr_FR';
import en_US from 'antd/lib/locale/en_US';
import de_DE from 'antd/lib/locale/de_DE';
import ar_EG from 'antd/lib/locale/ar_EG';
import he_IL from 'antd/lib/locale/he_IL';
import it_IT from 'antd/lib/locale/it_IT';
import pt_BR from 'antd/lib/locale/pt_BR';
import nl_NL from 'antd/lib/locale/nl_NL';
import es_ES from 'antd/lib/locale/es_ES';
import tr_TR from 'antd/lib/locale/tr_TR';
import zh_CN from 'antd/lib/locale/zh_CN';
import hi_IN from 'antd/lib/locale/hi_IN';
import ja_JP from 'antd/lib/locale/ja_JP';
import ko_KR from 'antd/lib/locale/ko_KR';
import ru_RU from 'antd/lib/locale/ru_RU';
import {SwapRightOutlined} from "@ant-design/icons";
import {Container, DateDepartFin, Explication, PaysView, Title, Voyage} from "./Components/Card.style";
import {CustomIcon} from "../boucleAssurer/Components/CustomIcon.style";

const langs = {
    "fr": {
        moment: "fr",
        locale: fr_FR
    },
    "en": {
        moment: "en-gb",
        locale: en_US
    },
    "de": {
        moment: "de",
        locale: de_DE
    },
    "ar": {
        moment: "ar",
        locale: ar_EG
    },
    "he": {
        moment: "he",
        locale: he_IL
    },
    "it": {
        moment: "it",
        locale: it_IT
    },
    "br": {
        moment: "br",
        locale: pt_BR
    },
    "nl": {
        moment: "nl",
        locale: nl_NL
    },
    "es": {
        moment: "es",
        locale: es_ES
    },
    "tu": {
        moment: "tr",
        locale: tr_TR
    },
    "zh": {
        moment: "zh",
        locale: zh_CN
    },
    "hi": {
        moment: "hi",
        locale: hi_IN
    },
    "ja": {
        moment: "ja",
        locale: ja_JP
    },
    "ru": {
        moment: "ru",
        locale: ru_RU
    }
};
moment.locale("fr");
const ApiContext = React.createContext({
    calculDevis: null,
    gtagLabel: ''
});

const App = ({lang, labels}) => {
    const [dateDepart, setDateDepart] = useState(() => {
        var date = window.localStorage.getItem("date-depart");
        if (!date) return moment().format("YYYY-MM-DD");
        return date;
    });
    const dateFinRef = React.useRef();
    window.boutonCalculer = React.useRef();
    const [dateFin, setDateFin] = useState(() => {
        var date = window.localStorage.getItem("date-fin");
        if (!date) return moment().format("YYYY-MM-DD");
        return date;
    });
    const [dateFinOpen, setDateFinOpen] = useState(false);
    const [recapDraw, setRecapDraw] = useState(false);
    const {calculDevis, gtagLabel} = React.useContext(ApiContext);

    var provenance = "";
    var provenance2code = "";
    var destination = "";
    var destination2code = "";

    if ( mc_souscription_data ) {
        if ( mc_souscription_data["provenance_pays-name"] !== undefined ) {
            provenance = mc_souscription_data["provenance_pays-name"];
        }
        if ( mc_souscription_data["provenance_pays"] !== undefined ) {
            provenance2code = mc_souscription_data["provenance_pays"].toLowerCase();
        }

        if ( mc_souscription_data["destination_pays-name"] !== undefined ) {
            destination = mc_souscription_data["destination_pays-name"];
        }
        if ( mc_souscription_data["destination_pays"] !== undefined ) {
            destination2code = mc_souscription_data["destination_pays"].toLowerCase();
        }
    }

    function disabledDateDepart(current) {
        // Can not select days before today and today
        return current && current < moment().startOf('day');
    }

    function disabledDateFin(current) {
        // Can not select days before today and today
        var dateDep;
        if (!dateDepart) dateDep = moment();
        else dateDep = moment(dateDepart);
        var datelimitavant = dateDep.format("YYYY-MM-DD");
        var datelimitapres = dateDep.add(365, "d").format("YYYY-MM-DD");

        return !current || current.isAfter(datelimitapres) || current.isSameOrBefore(datelimitavant);
    }

    function onChercheClick(e) {
        window.refreshCardProduit(document.getElementById("produits"), window.produits, "loading", window.onContinuerClick);
        setRecapDraw(true);
        calculDevis(dateDepart, dateFin).then(results => {
            // On actualise la card
            if (window.gtag && typeof window.gtag !== "undefined" ) {
                window.gtag('event', 'conversion', {'send_to': gtagLabel});
            }
            window.refreshCardProduit(document.getElementById("produits"), window.produits, "ready", window.onContinuerClick);
        });
    }

    function onEdit() {
        setRecapDraw(false);
    }

    function onDateDepartChange(date, dateString) {
        setDateDepart(dateString);
        dateFinRef.current.focus();
        setDateFinOpen(true);
    }

    function onDateFinChange(date, dateString) {
        setDateFin(dateString);
        setDateFinOpen(false);
    }

    function onProvenanceChange(value, info) {
        provenance2code = value;
        provenance = info.children;
        mc_souscription_data["provenance_pays"] = provenance2code.toUpperCase();
        mc_souscription_data["provenance_pays-name"] = provenance;
        window.localStorage.setItem("mc_souscription_data", JSON.stringify(mc_souscription_data));

        window.localStorage.setItem("provenance_pays", provenance2code.toUpperCase());
        window.localStorage.setItem("provenance_pays-name", provenance);
    }

    function onDestinationChange(value, info) {
        destination2code = value;
        destination = info.children;
        mc_souscription_data["destination_pays"] = destination2code.toUpperCase();
        mc_souscription_data["destination_pays-name"] = destination;
        window.localStorage.setItem("mc_souscription_data", JSON.stringify(mc_souscription_data));

        window.localStorage.setItem("destination_pays", destination2code.toUpperCase());
        window.localStorage.setItem("destination_pays-name", destination);
    }

    var lang1 = langs[lang];
    if (lang1.locale == "undefined") {
        lang1 = langs["en"];
    }
    const locale = lang1.locale;
    moment.locale(lang1.moment);
    var mDateDepart, mDateFin;
    if (!dateDepart) mDateDepart = moment();
    else mDateDepart = moment(dateDepart);
    if (!dateFin) mDateFin = moment();
    else mDateFin = moment(dateFin);

    const options = [];
    allCountries.forEach((pays, key) => {
        options.push(
            <Select.Option key={key} value={pays["iso2"].toString()}>{pays["name"]}</Select.Option>
        );
    });
    return (
        <ConfigProvider locale={locale}>
            { recapDraw && <Container>
                <Row>
                    <Col xs={24} md={17}>
                        <Row>
                            <Col xs={24} className="text-left">
                                <Title>
                                    <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/dates/images/calendar.png"} style={{
                                        width: 35,
                                        height: 35,
                                        padding: "4px"
                                    }} />{ labels.vos_informations } </Title>
                            </Col>
                        </Row>
                        <Row className="mt-2 mb-2">
                            <Col xs={24} md={12} className="text-left pl-1">
                                <PaysView>{ labels.pays_residence }: <b>{provenance}</b></PaysView>
                            </Col>
                            <Col xs={24} md={12} className="text-left">
                                <PaysView>{ labels.pays_destination }: <b>{destination}</b></PaysView>
                            </Col>
                        </Row>
                        <Row className="mt-2 mb-2">
                            <Col xs={24} md={12} className="text-left pl-1">
                                <PaysView>{ labels.date_depart }: <b>{ dateDepart }</b></PaysView>
                            </Col>
                            <Col xs={24} md={12} className="text-left">
                                <PaysView>{ labels.date_fin }: <b>{ dateFin }</b></PaysView>
                            </Col>
                        </Row>
                    </Col>
                    <Col xs={24} md={7} className="d-flex align-items-center justify-content-center border-left">
                        <Button type={"button"} className="universal-button" onClick={onEdit}>{labels.modifier}</Button>
                    </Col>
                </Row>
                </Container>}
            { !recapDraw && <Container>
                <Row>
                    <Col xs={24} className="text-left">
                        <Title>
                            <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/dates/images/calendar.png"} style={{
                            width: 35,
                            height: 35,
                            padding: "4px"
                        }} />{ labels.vos_informations }</Title>
                    </Col>
                </Row>
                <Row className="mt-2 mb-2">
                    <Col xs={24} md={12} className="text-left pl-1">
                        <PaysView>{ labels.pays_residence }: <Select
                            name="provenance_pays"
                            showSearch={true}
                            defaultValue={provenance2code}
                            filterOption={(input, option) =>
                                option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0
                                || option.props.value.toLowerCase().indexOf(input.toLowerCase()) >= 0
                            } style={{ width: "100%", padding: "0px 2px"}}
                            onChange={onProvenanceChange}
                        >{options}</Select></PaysView>
                    </Col>
                    <Col xs={24} md={12} className="text-left">
                        <PaysView>{ labels.pays_destination }: <Select
                            name="destination_pays"
                            showSearch={true}
                            defaultValue={destination2code}
                            filterOption={(input, option) =>
                                option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0
                                || option.props.value.toLowerCase().indexOf(input.toLowerCase()) >= 0
                            } style={{ width: "100%", padding: "0px 2px"}}
                            onChange={onDestinationChange}
                        >{options}</Select></PaysView>
                    </Col>
                </Row>
                <Row>
                    <Col xs={24} md={12} className="text-left">
                        <Voyage>{ labels.indiquer_date }: </Voyage>
                        <Explication>{ labels.text_indiquer_date } </Explication>
                    </Col>
                    <Col className="gutter-row" xs={24} md={12} style={{textAlign: "center", width: "100%"}}>
                        <Space direction={"vertical"}>
                            <Space direction={"horizontal"} className="w-100 mt-3">
                                <DateDepartFin>{ labels.date_depart }</DateDepartFin>
                                <span style={{ width: "15px" }}></span>
                                <DateDepartFin>{ labels.date_fin }</DateDepartFin>
                            </Space>
                            <Space direction={"horizontal"}>

                                <DatePicker disabledDate={disabledDateDepart} onChange={onDateDepartChange}
                                            value={mDateDepart}/>
                                <SwapRightOutlined/>
                                <DatePicker ref={dateFinRef} disabledDate={disabledDateFin} onChange={onDateFinChange}
                                            open={dateFinOpen} onClick={e => setDateFinOpen(true)} value={mDateFin}/>
                            </Space>

                            <Input type={"hidden"} name={"date-depart"} value={dateDepart}/>
                            <Input type={"hidden"} name={"date-fin"} value={dateFin}/>
                        </Space>
                    </Col>
                    <Col xs={24}>
                        <Button type={"button"} onClick={onChercheClick} className="universal-button"
                                ref={window.boutonCalculer}>{labels.calculer_le_devis}</Button>
                    </Col>
                </Row>
            </Container>}
        </ConfigProvider>
    )
}
window.showDatesForm = (element, lang, labels, options) => {
    var defaultValue = {
        calculDevis: options.calculDevis,
        gtagLabel: options.gtagLabel
    };
    ReactDom.render(
        <ApiContext.Provider value={defaultValue}>
            <App lang={lang} labels={labels}/>
        </ApiContext.Provider>
        , element);
}
