import {Col, Collapse, Row, Space} from "antd";
import React, {forwardRef, useCallback, useContext, useEffect, useImperativeHandle, useMemo, useRef} from "react";
import DatePickerApp from "./components/datepicker";
import LabelContext from "./context/label.context";
import FormuleChoisiCard from "./components/formules/App";
import GenerateCardProduit from "../generateCardProduit/App";
import FormRestant from "./components/formRestant";
import DetailContratContext from "./context/context";

const {Panel} = Collapse;

const DetailContrat = forwardRef((props, ref) => {
    const {
        options,
        initialData,
        contrat,
        dispatchContrat,
        produits,
        dispatchProduits,
        formuleChoisi,
        dispatchFormuleChoisi,
        dispatchInitialData,
        resteData,
        setTarifs,
        indiceProduit,
        setIndiceProduit,
        setResteData
    } = props;
    const ctx = useContext(DetailContratContext);
    const {calculDevis} = options;
    var refs = options.paysWidgets.map((item, i) => useRef());

    const onRechercheClick = (dateDepart, dateFin) => {
        var mDateDepart = moment(dateDepart),
            mDateFin = moment(dateFin);
        var diff = mDateFin.diff(mDateDepart, "d");
        dispatchContrat({
            type: "updateDuree",
            payload: diff
        })
        var dynamicParameters = {
            "date-depart": dateDepart,
            "date-fin": dateFin
        };

        var $ = window.jQuery;
        refs.forEach(ref => {
            var $parent = $(ref.current);
            const $select = $("select", $parent);
            dynamicParameters[$select.attr("name")] = $select.val();
        });
        // loading
        calculDevis(dynamicParameters, initialData).then(results => {
            const tarification = [...results.filter((res) => {
                const produitID = formuleChoisi.produit.produitID;
                const garanties = formuleChoisi.garanties;
                var trouve = true;
                // on compare tous les paramètres du devis à ceux des garanties
                garanties.forEach(garantie => {
                    trouve = trouve && (res[garantie.code_garantie] == garantie.valeur);
                });
                return (produitID == res._wpcf7) && trouve;
            })].pop();
            if (tarification) {
                dispatchInitialData({
                    type: "setState",
                    payload: tarification
                });
            }
            const newProduits = produits.map(groupe => {
                const produitID = groupe.produit.produitID;
                const garanties = groupe.garanties;
                var result = null;
                results.forEach(res => {
                    if (result != null) return;
                    if (produitID != res._wpcf7) return;
                    var trouve = true;
                    // on compare tous les paramètres du devis à ceux des garanties
                    garanties.forEach(garantie => {
                        trouve = trouve && (res[garantie.code_garantie] == garantie.valeur);
                    });
                    if (trouve) {
                        result = res;
                    }
                });
                if(result) {
                    groupe.produit.tarif = result["devis-st"];
                    groupe.produit.formated_tarif = result["formated-devis-st"];
                }
                return groupe;
            });
            const p = [...newProduits.filter((pr) => {
                return pr.formule.ID == formuleChoisi.formule.ID;
            })].pop();
            dispatchFormuleChoisi({
                type: "updateDevis",
                payload: p.produit.tarif
            })
            dispatchContrat({
                type: "updateDevis",
                payload: p.produit.tarif
            })
            dispatchProduits({
                type: "update",
                payload: newProduits
            })
            setTarifs(results);
            // Si edition contrat (showContrat)
            // comparaison ancien tarif et nouveau tarif pour la formule choisi
            // d = diff(tarification, options.tarifs[indiceProduit])
            // si d > 0  alors  on affiche une page de paiement de la différence
            // si d = 0 alors on ne fait rien
            // sinon on bloque l'enregistrement
        })
    };
    const datePickerRef = useRef();
    useImperativeHandle(ref, () => {
        return {
            calculerDevis() {
                datePickerRef.current.onCalculerDevis();
            }
        }
    })
    useEffect(() => {
        dispatchContrat({
            type: "updateDevis",
            payload: formuleChoisi.produit.tarif
        })
        const shortcodesGaranties = formuleChoisi.garanties.map((shortcode) => {
            return shortcode.code_garantie;
        });
        var importantParameters = ["date-depart", "date-fin", "_wpcf7", "personne", "formuleChoisi", "provenance_pays", "destination_pays"]
        const newData = {};
        for (var key in options.initialData) {
            var value = options.initialData[key];
            const notGaranties = shortcodesGaranties.indexOf(key) == -1;
            const notImportant = importantParameters.indexOf(key) == -1;
            if (notGaranties && notImportant) {
                newData[key] = value;
            }
        }
        newData["_wpcf7"] = formuleChoisi.produit.produitID;
        setResteData(newData);
    }, [formuleChoisi]);
    const onChoixFormule = useCallback((produit, indice) => {
        setIndiceProduit(indice);
        dispatchFormuleChoisi({
            type: "setProduit",
            payload: produit
        })
    }, []);
    const hiddenFields = [];
    for (var key in resteData) {
        var value = resteData[key];
        hiddenFields.push(<input key={key} name={key} value={value} type={"hidden"}/>)
    }
    return (
        <Row>
            <Col span={24}>
                <Row>
                    <Col span={24}>
                        <DatePickerApp dateDepartprop={options.dateDepart}
                                       dateFinprop={options.dateFin}
                                       labels={options.labels}
                                       ref={datePickerRef}
                                       lang={options.lang} onRechercheClick={onRechercheClick}/>
                    </Col>
                </Row>
                <Row>
                    <Col span={24}>
                        <Space direction={"vertical"} style={{marginBottom: "8px"}}>
                            <Space>
                                {options.paysWidgets.map((payWidget, i) => (
                                    <div key={i} dangerouslySetInnerHTML={{__html: payWidget}} ref={refs[i]}/>
                                ))}
                            </Space>
                        </Space>
                    </Col>
                </Row>
                <Row>
                    <Col span={24}>
                        <h3>{ctx.formule_applique}</h3>
                        <LabelContext.Provider value={{labels: options.labels}}>
                            <FormuleChoisiCard product={formuleChoisi}/>
                        </LabelContext.Provider>
                    </Col>
                </Row>
                <Row>
                    <Col span={24}>
                        <Collapse>
                            <Panel header={ctx.changer_formule}>
                                <GenerateCardProduit data={produits} onClick={onChoixFormule}/>
                            </Panel>
                        </Collapse>
                    </Col>
                </Row>
                <Row>
                    <Col span={24}>
                        <FormRestant data={contrat}/>
                    </Col>
                </Row>
                {hiddenFields}
            </Col>
        </Row>
    )
})

export default DetailContrat;