import React, {useEffect, useReducer, useRef, useState} from 'react';
import DetailContrat from "./DetailContrat";
import SouscripteurBeneficiaire from "../souscripteurBeneficiaires/App";
import ContratDataReducer from "./reducers/contratData.reducer";
import ProduitsReducer from "./reducers/produits.reducer";
import ProduitReducer from "./reducers/produit.reducer";
import InitialDataReducer from "./reducers/initialData.reducer";
import DetailContratContext from "./context/context";
import CCard from "../souscripteurBeneficiaires/components/CCard";

const DevisEdit = ({contratData, options}) => {
    // Contrat information
    const [contrat, dispatchContrat] = useReducer(ContratDataReducer, contratData);
    const [produits, dispatchProduits] = useReducer(ProduitsReducer, options.produits);
    const [formuleChoisi, dispatchFormuleChoisi] = useReducer(ProduitReducer, options.formuleChoisi);
    // Data ampiasaina @ tarification
    const [initialData, dispatchInitialData] = useReducer(InitialDataReducer, options.initialData);
    const [tarifs, setTarifs] = useState(() => options.tarifs);
    const [indiceProduit, setIndiceProduit] = useState(() => options.indiceFormuleChoisi);
    const [resteData, setResteData] = useState(() => options.initialData);
    // Assure information
    const [listAssure, setListAssure] = useState(options.souscripteurBeneficiairesData.liste);
    const detailContratRef = useRef();
    const onUpdateAssuresList = React.useCallback((personnes) => {
        async function dispatch() {
            await dispatchInitialData({
                type: "setPersonnes",
                payload: personnes
            })
            // recalcul tarif
            detailContratRef.current.calculerDevis();
        }

        dispatch();
    });

    const fetchCountry = async (alpha2code) => {
        const responsePromise = await fetch(`${window.site_url}/fresh/pays_name/${alpha2code}`);
        return await responsePromise.json();
    }

    useEffect(() => {
        async function fetchData() {
            if (initialData) {
                var newListAssure = {...listAssure};
                for (var idClient in newListAssure) {
                    const assure = newListAssure[idClient];
                    const personne = {...initialData.personne[idClient]};
                    const alpha2code = personne["personne--nationalite"];
                    const nationalite = {...assure.assurePrincipal["personne--nationalite"]};
                    nationalite.alpha2code = alpha2code;
                    var countryName = alpha2code;
                    if (typeof alpha2code != "undefined") countryName = await fetchCountry(alpha2code);
                    nationalite.countryName = countryName;
                    delete personne["personne--nationalite"];
                    assure.assurePrincipal = {...personne};
                    assure.assurePrincipal["personne--nationalite"] = {...nationalite};
                    newListAssure[idClient].assurePrincipal = {...assure.assurePrincipal};
                }
                setListAssure({...newListAssure});
            }
        }

        fetchData();
    }, [initialData])
    useEffect(() => {
        //find index ici
        const payload = tarifs[indiceProduit];
        dispatchInitialData({
            type: "setState",
            payload: {...payload}
        })
    }, [formuleChoisi, tarifs])
    return (
        <>
            <DetailContratContext.Provider value={options.labels}>
                <CCard>
                    <DetailContrat
                        options={options}
                        initialData={initialData}
                        contrat={contrat}
                        dispatchContrat={dispatchContrat}
                        produits={produits}
                        dispatchProduits={dispatchProduits}
                        formuleChoisi={formuleChoisi}
                        dispatchFormuleChoisi={dispatchFormuleChoisi}
                        ref={detailContratRef}
                        dispatchInitialData={dispatchInitialData}
                        resteData={resteData}
                        setTarifs={setTarifs}
                        indiceProduit={indiceProduit}
                        setIndiceProduit={setIndiceProduit}
                        setResteData={setResteData}/>
                </CCard>
            </DetailContratContext.Provider>
            <SouscripteurBeneficiaire options={options} listAssure={listAssure} setListAssure={setListAssure}
                                      onUpdateAssuresList={onUpdateAssuresList}/>
        </>
    )
};

export default DevisEdit;