import React, {useState, memo, forwardRef, useImperativeHandle, useEffect} from "react"
import {DatePicker, Row, Col, Space, ConfigProvider, Input, Button} from "antd"
import 'antd/dist/antd.css';
import moment from "moment";

import('moment/locale/fr');
import fr_FR from 'antd/lib/locale/fr_FR';
import en_US from 'antd/lib/locale/en_US';
import de_DE from 'antd/lib/locale/de_DE';
import ar_EG from 'antd/lib/locale/ar_EG';
import he_IL from 'antd/lib/locale/he_IL';
import it_IT from 'antd/lib/locale/it_IT';
import pt_BR from 'antd/lib/locale/pt_BR';
import nl_NL from 'antd/lib/locale/nl_NL';
import es_ES from 'antd/lib/locale/es_ES';
import {SwapRightOutlined} from "@ant-design/icons";
import ApiContext from "../context/ApiContext";

const langs = {
    "fr": {
        moment: "fr",
        locale: fr_FR
    },
    "en": {
        moment: "en-gb",
        locale: en_US
    },
    "de": {
        moment: "de",
        locale: de_DE
    },
    "ar": {
        moment: "ar",
        locale: ar_EG
    },
    "he": {
        moment: "he",
        locale: he_IL
    },
    "it": {
        moment: "it",
        locale: it_IT
    },
    "br": {
        moment: "br",
        locale: pt_BR
    },
    "nl": {
        moment: "nl",
        locale: nl_NL
    },
    "es": {
        moment: "es",
        locale: es_ES
    }
};
moment.locale("fr");

const DatePickerApp = memo(forwardRef(({lang, labels, onRechercheClick, dateDepartprop = null, dateFinprop = null}, ref) => {
    const [dateDepart, setDateDepart] = useState(() => {
        if (dateDepartprop) {
            if (moment(dateDepartprop).isValid()) return dateDepartprop;
            return moment(dateDepartprop, "DD/MM/YYYY").format("YYYY-MM-DD")
        }
        var date = window.sessionStorage.getItem("date-depart");
        if (!date) return moment().format("YYYY-MM-DD");
        return date;
    });
    const dateFinRef = React.useRef();
    const [dateFin, setDateFin] = useState(() => {
        if (dateFinprop) {
            if (moment(dateFinprop).isValid()) return dateFinprop;
            return moment(dateFinprop, "DD/MM/YYYY").format("YYYY-MM-DD")
        }
        var date = window.sessionStorage.getItem("date-fin");
        if (!date) return moment().format("YYYY-MM-DD");
        return date;
    });
    const [dateFinOpen, setDateFinOpen] = useState(false);

    useEffect(() => {
        onRechercheClick(dateDepart, dateFin);
    }, [dateDepart, dateFin]);
    function disabledDateDepart(current) {
        // Can not select days before today and today
        return current && current < moment().startOf('day');
    }

    function disabledDateFin(current) {
        // Can not select days before today and today
        var dateDep;
        if (!dateDepart) dateDep = moment();
        else dateDep = moment(dateDepart);
        return current && current < dateDep.startOf('day');
    }

    useImperativeHandle(ref, () => {
        return {
            onCalculerDevis(){
                onRechercheClick(dateDepart,dateFin);
            }
        }
    })
    function onDateDepartChange(date, dateString) {
        setDateDepart(dateString);
        dateFinRef.current.focus();
        setDateFinOpen(true);
    }

    function onDateFinChange(date, dateString) {
        setDateFin(dateString);
        setDateFinOpen(false);
    }

    const locale = langs[lang].locale;
    moment.locale(langs[lang].moment);
    var mDateDepart, mDateFin;
    if (!dateDepart) mDateDepart = moment();
    else {
        mDateDepart = moment(dateDepart);
        if (!mDateDepart.isValid()) {
            mDateDepart = moment(dateDepart, "DD/MM/YYYY")
        }
    }
    if (!dateFin) mDateFin = moment();
    else {
        mDateFin = moment(dateFin);
        if (!mDateFin.isValid()) {
            mDateFin = moment(dateFin, "DD/MM/YYYY")
        }
    }
    return (
        <ConfigProvider locale={locale}>
            <Row>
                <Col className="gutter-row" sm={24} md={24}
                     style={{textAlign: "left", width: "100%"}}>
                    <Space direction={"vertical"}>
                        <Space direction={"horizontal"}>
                            <DatePicker disabledDate={disabledDateDepart}
                                        onChange={onDateDepartChange}
                                        value={mDateDepart}/>
                            <SwapRightOutlined/>
                            <DatePicker ref={dateFinRef}
                                        disabledDate={disabledDateFin}
                                        onChange={onDateFinChange}
                                        open={dateFinOpen}
                                        onClick={e => setDateFinOpen(true)}
                                        value={mDateFin}/>
                        </Space>

                        <Input type={"hidden"} name={"date-depart"} value={dateDepart}/>
                        <Input type={"hidden"} name={"date-fin"} value={dateFin}/>
                    </Space>
                </Col>
            </Row>
        </ConfigProvider>
    )
}));
export default DatePickerApp;