import {Label, Title} from './Components/Header.style';
import {Plan, Price, Quote, List, ListElement, Link} from './Components/Pricing.style';
import {FontAwesomeIcon} from '@fortawesome/react-fontawesome';
import {faCheckCircle, faTimesCircle, faFilePdf} from '@fortawesome/free-solid-svg-icons';
import React, {useContext, useMemo} from 'react';
import LabelContext from "../../context/label.context";


class Garantie extends React.Component {
    render() {
        const {garantie} = this.props
        const icon = garantie.icone == "include-icon" ?
            <FontAwesomeIcon icon={faCheckCircle} color='#6ab04c' style={{marginRight: '4px'}}/> :
            <FontAwesomeIcon icon={faTimesCircle} color='#eb4d4b' style={{marginRight: '4px'}}/>
        return <ListElement> {icon} {garantie.label} </ListElement>
    }
}

class File extends React.Component {
    render() {
        const {file} = this.props
        return <Link href={file.downloadUrl} target="_blank" className="mt-2 w-100"><FontAwesomeIcon
            icon={faFilePdf}/> {file.title} </Link>
    }
}

const Card = ({product}) => {
    const {labels} = useContext(LabelContext);
    const optionGarantie = []
    const listfiles = []
    product.garanties.forEach((garantie, index) => {
        optionGarantie.push(<Garantie key={index} garantie={garantie}/>)
    })
    if (product.files && product.files.length > 0) {
        product.files.forEach((file, index) => {
            listfiles.push(<File key={index} file={file}/>)
        })
    }
    const hiddenFields = useMemo(() => product.garanties.map((shortcode, i) => {
        return <input type={"hidden"} key={i} name={shortcode.code_garantie} value={shortcode.valeur}/>
    }));
    return (
        <Plan>
            <Label htmlFor={"option-" + product.formule.ID}>
                <input type={"hidden"} id={"option-" + product.formule.ID} name="formuleChoisi"
                       value={product.formule.ID}/>
                <Title> {product.produit.label} </Title>
                <Price>  {product.formule.label} </Price>
                <Quote>{labels.a_partir_de} {product.produit.tarif} {product.produit.currency}</Quote>
                <List>
                    {optionGarantie}
                </List>
                {listfiles}
            </Label>
            {hiddenFields}
        </Plan>
    );
}

function App({product}) {
    return (
        <Card product={product}/>
    );
}

export default App;
