import {
    Container,
    ContainerButtonPrice,
    ContainerContact,
    ContainerFiles,
    ContainerPop
} from './Components/Container.style';
import {Label, Title} from './Components/Header.style';
import {
    Pricing,
    Radio,
    Plan,
    Price,
    Quote,
    List,
    ListElement,
    Link,
    PElement,
    Detail
} from './Components/Pricing.style';
import {FontAwesomeIcon} from '@fortawesome/react-fontawesome';
import {faCheckCircle, faTimesCircle, faFilePdf} from '@fortawesome/free-solid-svg-icons';
import React from 'react';
import 'antd/dist/antd.css';
import {Popover, Button as AntButton, Divider, Form, Input, Row, Col, Space} from 'antd';
import LabelContext from "../context/label.context";
import Ribbon from "./Components/Ribbon";
import {CustomIcon} from "../boucleAssurer/Components/CustomIcon.style";


class Garantie extends React.Component {
    render() {
        const {garantie} = this.props;
        const icon = garantie.icone == "include-icon" ?
            <FontAwesomeIcon icon={faCheckCircle} color='#6ab04c' style={{margin: '4px', width: '1.5em', overflow:'unset'}}/> :
            <FontAwesomeIcon icon={faTimesCircle} color='#eb4d4b' style={{margin: '4px', width: '1.5em', overflow:'unset'}}/>
        return garantie.show === 'true' ? <ListElement> {icon} <PElement>{garantie.label}</PElement> </ListElement> : ''
    }
}

export class File extends React.Component {
    render() {
        const {file} = this.props;
        return <Link href={file.downloadUrl} target="_blank" className="mt-2 w-100"><FontAwesomeIcon
            icon={faFilePdf}/> {file.title} </Link>
    }
}

class Card extends React.Component {
    static contextType = LabelContext;

    constructor(props) {
        super(props);
        this.onClick = this.onClick.bind(this);
        this.onSeconnecterClick = this.onSeconnecterClick.bind(this);
        this.downloadAllPDF = this.downloadAllPDF.bind(this);
        this.contactClicked = this.contactClicked.bind(this);
        this.handleWindowSizeChange = this.handleWindowSizeChange.bind(this);
        this.setWidth = this.setWidth.bind(this);
    }

    state = {
        visible: false,
        files: [],
        width: window.innerWidth
    }

    setWidth = ( width ) => {
        this.setState({
            width: width
        });
    }

    handleWindowSizeChange = () => {
        this.setWidth(window.innerWidth);
    }

    componentWillMount = () => {
        window.addEventListener('resize', this.handleWindowSizeChange);
    }

    componentWillUnmount = () => {
        window.removeEventListener('resize', this.handleWindowSizeChange);
    }

    hide = () => {
        this.setState({
            visible: false,
        });
    }

    handleVisibleChange = visible => {
        this.setState({visible});
    }

    onClick(e) {
        this.props.onClick(e);
        this.hide();
    }

    onSeconnecterClick(e) {
        this.props.onSeconnecterClick(e);
    }

    downloadAllPDF(e) {
        this.state.files.forEach((file, index) => {
            var link = document.createElement('a');
            link.href = file.props.file.downloadUrl;
            link.download = file.props.file.title;
            link.dispatchEvent(new MouseEvent('click'));
        });
    }

    contactClicked( contact ) {
        var link = document.createElement('a');
        switch ( contact ) {
            case "phone":
                link.href = "tel:+33182865626";
                link.dispatchEvent(new MouseEvent('click'));
                break;
            case "messenger":
                link.href = "http://m.me/Mondialcare.eu";
                link.dispatchEvent(new MouseEvent('click'));
                break;
            case "whatsapp":
                link.href = "https://wa.me/message/EPDALC5H7WJEG1";
                link.dispatchEvent(new MouseEvent('click'));
                break;
            default:
                break;
        }
    }

    render() {
        const {product} = this.props
        const {labels} = this.context;
        const optionGarantieleft = [];
        const optionGarantieright = [];
        const listfiles = []
        const nbrGarantie = product.garanties.length;
        const moitienbr = nbrGarantie / 2;
        const contentPopover = (
            <ContainerPop>
                <AntButton type={"button"} className="universal-button" style={{width: "100%"}}
                           onClick={this.onSeconnecterClick}>{labels.loginText}</AntButton>
                <Divider dashed> {labels.ou} </Divider>
                <AntButton type={"button"} onClick={this.onClick} htmlType={"submit"}
                           style={{width: "100%"}}>{labels.inviteText}</AntButton>
            </ContainerPop>
        );

        const isMobile = this.state.width <= 768;

        product.garanties.sort((a, b) => {
            return a.order - b.order;
        }).forEach((garantie, index) => {
            if ( index <= moitienbr ) {
                optionGarantieleft.push(<Garantie key={index} garantie={garantie}/>)
            } else {
                optionGarantieright.push(<Garantie key={index} garantie={garantie}/>)
            }
        });
        product.files.forEach((file, index) => {
            listfiles.push(<File key={index} file={file}/>)
        })
        this.state.files = listfiles;

        var afterLogin = "false";
        if (window.localStorage.getItem("after_login") !== null) {
            afterLogin = window.localStorage.getItem("after_login");
        }


        return (
            <React.Fragment>
                <Plan>
                    {product.miseEnAvant && product.miseEnAvant === "true" && (
                        <Ribbon className={"ribbon"} background={"rgb(255, 102, 0)"}>
                            <span>{labels.populaire}</span>
                        </Ribbon>
                    )}
                    <Label htmlFor={"option-" + product.formule.ID}>
                        <Row>
                            <Col xs={24} md={17} className="garantie-content">
                                <Radio id={"option-" + product.formule.ID} type="radio" name="formule"
                                       value={product.formule.ID}/>
                                <Title> {product.produit.label} </Title>
                                <Price className="formule-title">  {product.formule.label} </Price>
                                <Row>
                                    { isMobile && <Col xs={24} className="text-center">
                                        <Quote>{labels.a_partir_de} {product.produit.tarif} {product.produit.currency}</Quote>
                                        <ContainerButtonPrice style={{ marginBottom: "15px" }}>
                                            {afterLogin == "false" && (
                                                <Popover
                                                    content={ contentPopover }
                                                    trigger="click"
                                                    visible={this.state.visible}
                                                    onVisibleChange={this.handleVisibleChange}

                                                >
                                                    <AntButton type={"button"}
                                                               className="universal-button"> {product.subscribeButton.label} </AntButton>
                                                </Popover>
                                            )}
                                            {afterLogin == "true" && (
                                                <AntButton type={"button"} className="universal-button"
                                                           onClick={this.onClick}> {product.subscribeButton.label} </AntButton>
                                            )}
                                        </ContainerButtonPrice>
                                        <span style={{ fontSize: "15px" }}>{labels.question}</span>
                                        <Row style={{ margin: "5px"}}>
                                            <Col xs={8} onClick={() => this.contactClicked('phone')} style={{ cursor: "pointer"}}>
                                                <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/generateCardProduit/images/phone.png"} style={{
                                                    width: 55,
                                                    height: 55,
                                                    padding: "4px"
                                                }} />
                                            </Col>
                                            <Col xs={8} onClick={() => this.contactClicked('messenger')} style={{ cursor: "pointer"}}>
                                                <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/generateCardProduit/images/messenger.png"} style={{
                                                    width: 55,
                                                    height: 55,
                                                    padding: "4px"
                                                }} />
                                            </Col>
                                            <Col xs={8} onClick={() => this.contactClicked('whatsapp')} style={{ cursor: "pointer"}}>
                                                <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/generateCardProduit/images/whatsapp.png"} style={{
                                                    width: 55,
                                                    height: 55,
                                                    padding: "4px"
                                                }} />
                                            </Col>
                                        </Row>
                                    </Col>}
                                </Row>
                                <Row className="list-garantie-content">
                                    <Col xs={24} md={12} style={{ paddingRight: '18px' }}>
                                        {optionGarantieleft}
                                    </Col>
                                    <Col xs={24} md={12} style={{ paddingRight: '18px' }}>
                                        {optionGarantieright}
                                    </Col>
                                </Row>
                            </Col>
                            <Col xs={0} md={1} style={{ borderLeft: '2px solid black' }} className="separation"></Col>
                            <Col xs={24} md={6} style={{ display: 'flex', justifyContent: 'space-between', flexDirection: 'column'}} className="block-info">
                                <ContainerFiles>
                                    <Detail> {labels.detail_formule} </Detail>
                                    {listfiles}
                                </ContainerFiles>
                                <ContainerContact>
                                    <Row>
                                        <Col xs={8} onClick={() => this.contactClicked('phone')} style={{ cursor: "pointer"}}>
                                            <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/generateCardProduit/images/phone.png"} style={{
                                                width: 55,
                                                height: 55,
                                                padding: "4px"
                                            }} />
                                        </Col>
                                        <Col xs={8} onClick={() => this.contactClicked('messenger')} style={{ cursor: "pointer"}}>
                                            <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/generateCardProduit/images/messenger.png"} style={{
                                                width: 55,
                                                height: 55,
                                                padding: "4px"
                                            }} />
                                        </Col>
                                        <Col xs={8} onClick={() => this.contactClicked('whatsapp')} style={{ cursor: "pointer"}}>
                                            <CustomIcon src={window.parent_site + "/wp-content/plugins/wp-lumen-plugin-framework-master/resources/assets/js/generateCardProduit/images/whatsapp.png"} style={{
                                                width: 55,
                                                height: 55,
                                                padding: "4px"
                                            }} />
                                        </Col>
                                    </Row>
                                    <Row>
                                        <Col className="text-center">
                                            <span style={{ fontSize: "15px"}}>{labels.question}</span>
                                        </Col>
                                    </Row>
                                </ContainerContact>
                                { !isMobile && <ContainerButtonPrice>
                                    <Quote>{labels.a_partir_de} {product.produit.tarif} {product.produit.currency}</Quote>
                                    {afterLogin == "false" && (
                                        <Popover
                                            content={ contentPopover }
                                            trigger="click"
                                            visible={this.state.visible}
                                            onVisibleChange={this.handleVisibleChange}

                                        >
                                            <AntButton type={"button"}
                                                       className="universal-button"> {product.subscribeButton.label} </AntButton>
                                        </Popover>
                                    )}
                                    {afterLogin == "true" && (
                                        <AntButton type={"button"} className="universal-button"
                                                   onClick={this.onClick}> {product.subscribeButton.label} </AntButton>
                                    )}
                                </ContainerButtonPrice>}
                            </Col>
                            <Col xs={24} className="block-mobile text-center">
                                <Link className="mt-2 mb-2 w-100" onClick={ this.downloadAllPDF }><FontAwesomeIcon
                                    icon={faFilePdf}/> {labels.telecharger_doc} </Link>
                            </Col>
                        </Row>
                    </Label>
                </Plan>
            </React.Fragment>
        )
    }
}

function CardList({products, onClick, onSeconnecterClick}) {
    const cardRow = [];

    products.forEach((product, index) => {
        cardRow.push(<Card key={index} product={product} onClick={e => onClick(product, index)}
                           onSeconnecterClick={e => onSeconnecterClick(product)}/>);
    })

    return <Container>
        <Pricing>
            {cardRow}
        </Pricing>
    </Container>
}

class CardContainer extends React.Component {
    state = {
        loginVisible: false,
        product: []
    }
    static contextType = LabelContext;

    constructor(props) {
        super(props);
        this.toggleLoginForm = this.toggleLoginForm.bind(this);
        this.onFinish = this.onFinish.bind(this);
        this.onBackClicked = this.onBackClicked.bind(this);
    }

    componentDidMount() {
        this.setState({
            loginVisible: false,
            cardVisible: true,
            product: []
        })
    }

    toggleLoginForm(product) {
        const {loginVisible} = this.state;
        var $ = window.jQuery;
        if (!loginVisible) {
            $("#datesform").hide();
        } else {
            $("#datesform").show();
        }
        this.setState({
            loginVisible: !loginVisible,
            product: product
        })
    }

    onFinish(values) {
        var $ = jQuery;
        var updateQueryStringParameter = function (uri, key, value) {
            var re = new RegExp("([?&])" + key + "=.*?(&|$)", "i");
            var separator = uri.indexOf('?') !== -1 ? "&" : "?";
            if (uri.match(re)) {
                return uri.replace(re, '$1' + key + "=" + value + '$2');
            } else {
                return uri + separator + key + "=" + value;
            }
        }
        $.ajax({
            type: "post",
            url: window.loginUrl,
            data: values,
            success: (res) => {
                if (res.status === "success") {
                    window.localStorage.setItem("after_login", "true");
                    window.localStorage.setItem("info-souscripteur", JSON.stringify(res.meta));
                    var href = window.location.href;
                    href = updateQueryStringParameter(href, "date-depart", window.localStorage.getItem("date-depart"));
                    href = updateQueryStringParameter(href, "date-fin", window.localStorage.getItem("date-fin"));
                    this.props.onClick(this.state.product);
                    window.location.href = href;
                } else {
                    alert(res.message);
                }
            }
        });
        console.log('Success:', values);
    }

    onBackClicked() {
        var $ = window.jQuery;
        $("#datesform").show();
        this.setState({
            loginVisible: false
        });
    }

    render() {
        const {products} = this.props
        const {loginVisible} = this.state;
        const {labels} = this.context;
        if (loginVisible) return (
            <div>
                <h3 className="text-center">{labels.login_title}</h3>
                <Form
                    name="basic"
                    labelCol={{
                        span: 12,
                    }}
                    wrapperCol={{
                        span: 24,
                    }}
                    layout='vertical'
                    initialValues={{
                        remember: true,
                    }}
                    onFinish={this.onFinish}
                    autoComplete="off"
                    style={{
                        margin: "auto",
                        maxWidth: 450
                    }}
                >
                    <Form.Item
                        name="_ajax_nonce"
                        initialValue={window.nonce}
                    >
                        <Input type={"hidden"}/>
                    </Form.Item>
                    <Row>
                        <Col xs={24}>
                            <Form.Item
                                label={labels.username}
                                name="username"
                                rules={[
                                    {
                                        required: true,
                                        message: labels.usernameMsg,
                                    },
                                ]}
                            >
                                <Input/>
                            </Form.Item>
                        </Col>
                        <Col xs={24}>
                            <Form.Item
                                label={labels.password}
                                name="password"
                                rules={[
                                    {
                                        required: true,
                                        message: labels.passwordMsg,
                                    },
                                ]}
                            >
                                <Input.Password/>
                            </Form.Item>
                        </Col>
                        <Col xs={24} style={{textAlign: "right"}}>
                            <Space>
                                <AntButton type="primary" htmlType="button" type={"ghost"} style={{
                                    borderRadius: ".25rem",
                                    marginBottom: "8px",
                                    cursor: "pointer"
                                }} onClick={this.onBackClicked}>
                                    {labels.backText ? labels.backText : "Back"}
                                </AntButton>
                                <AntButton type="primary" htmlType="submit" style={{
                                    background: "#ff6600",
                                    color: "#fff",
                                    border: "1px solid #ff6600",
                                    borderRadius: ".25rem",
                                    marginBottom: "8px",
                                    cursor: "pointer"
                                }}>
                                    {labels.loginText}
                                </AntButton>
                            </Space>
                        </Col>
                    </Row>
                </Form>
            </div>
        );
        return <CardList products={products} onClick={this.props.onClick} onSeconnecterClick={this.toggleLoginForm}/>
    }
}

function App({data, onClick}) {
    return (
        <CardContainer products={data} onClick={onClick}/>
    );
}

export default App;