import {Card, Col, Collapse, Divider, Radio, Typography, Button, Space, Checkbox} from "antd";
import {CheckOutlined} from "@ant-design/icons";
import React from "react";

const {Panel} = Collapse;

const CardGroupeProduit = ({item, shortcodes, onItemChange, namePrefix, onAssureChange, onSouscripteurChange, onCardRemove, onChangeMiseEnAvant}) => {
    if (typeof item.miseEnAvant == "undefined") {
        item.miseEnAvant = false;
    }
    return (
        <Col span={8}>
            <Card title={item.produit.label}
                  extra={<Space direction={"horizontal"}><span>{item.produit.onglet}</span><Button type={"primary"}
                                                                                                   onClick={onCardRemove}>Enlever</Button></Space>}>
                <Checkbox onChange={e => onChangeMiseEnAvant(e.target.checked)}
                          checked={item.miseEnAvant} name={namePrefix + "[miseEnAvant][" + item.formule.ID + "]"}
                          value={true}>Mettre en avant</Checkbox><br/>
                <Typography>
                    {item.formule.label}
                </Typography>
                <Collapse defaultActiveKey={['1']}>
                    <Panel header={"Garanties"} key={'1'}>
                        <ul>
                            {item.garanties.map((garantie, index) => (
                                <li key={index}><CheckOutlined
                                    style={{color: "green"}}/> {garantie.label} (<b>{garantie.valeur}</b>)
                                </li>
                            ))}
                        </ul>
                    </Panel>
                </Collapse>

                <Divider>Liste des assurés</Divider>

                <Radio.Group name={namePrefix + "[widgetAssure][" + item.formule.ID + "]"}
                             defaultValue={item.widgetAssure} onChange={e => onAssureChange(e.target.value)}
                             buttonStyle="outline">
                    <Radio.Button value="individuel">Individuel</Radio.Button>
                    <Radio.Button value="famille">Famille</Radio.Button>
                </Radio.Group>

                <Divider>Le souscripteur est</Divider>

                <Radio.Group name={namePrefix + "[widgetSouscripteur][" + item.formule.ID + "]"}
                             defaultValue={item.widgetSouscripteur} onChange={e => onSouscripteurChange(e.target.value)}
                             buttonStyle="outline">
                    <Radio.Button value="personne">Une personne</Radio.Button>
                    <Radio.Button value="entreprise">Une entreprise</Radio.Button>
                </Radio.Group>

                <Divider>Shortcodes</Divider>

                <ul>
                    {shortcodes.map((shortcode, ind) => {
                        var isChecked = false;
                        if (item.shortcodes && typeof item.shortcodes != "undefined") isChecked = item.shortcodes.indexOf(shortcode.shortcode) !== -1
                        return (
                            <li key={ind}><input type={"checkbox"}
                                                 checked={isChecked}
                                                 value={shortcode.shortcode}
                                                 onChange={e => onItemChange(ind, e.target, item)}/> {shortcode.shortcode}
                            </li>
                        )
                    })}
                </ul>
                <input type={"hidden"} name={namePrefix + "[shortcodes][" + item.formule.ID + "]"}
                       value={JSON.stringify(item.shortcodes)}/>

            </Card>
        </Col>
    )
}
export default CardGroupeProduit;