import React, {useContext, useEffect, useState} from 'react';

import LabelContext from "../context";

import {
    Col,
    Collapse, ConfigProvider,
    Form,
    Popconfirm
} from "antd";

import FormAssure from "./FormAssure";

import CModal from "../components/CModal";
import CCard from "../components/CCard";
import CCollapse from "../components/CCollapse";
import CButton from "../components/CButton";
import DivFlex from "../components/DivFlex";
import CRow from "../components/CRow";
import frLocale from "antd/es/date-picker/locale/fr_FR";
import Personne from "../../boucleAssurer/personne/personne";

const {Panel} = Collapse;

const AssureInfo = ({options, listAssure, setListAssure, onUpdateAssuresList}) => {
    const ctx = useContext(LabelContext);

    const [form] = Form.useForm();

    const [showModal, setShowModal] = useState(false);
    const [dataAssure, setDataAssure] = useState(() => ({
        isEdit: false,
        assure: {}
    }));
    const [index, setIndex] = useState();
    const [personnes, setPersonnes] = useState([]);
    let [inputHiddenPersonne, setInputHiddenPersonne] = useState([]);

    function buildListPersonnes(listPers) {
        inputHiddenPersonne = [];
        Object.entries(listAssure).map((assures, key) => {
            const index = assures[0];
            const assure = assures[1];
            var datenaiss = assure.assurePrincipal["personne--date-naiss"];
            if (moment.isMoment(datenaiss)) datenaiss = datenaiss.format("DD/MM/YYYY");
            const personne = {
                "personne--indice": index,
                "personne--nom": assure.assurePrincipal["personne--nom"],
                "personne--prenom": assure.assurePrincipal["personne--prenom"],
                "personne--date-naiss": datenaiss,
                "personne--nationalite": assure.assurePrincipal["personne--nationalite"].alpha2code,
                "personne--passeport": assure.assurePrincipal["personne--passeport"]
            };
            listPers[index] = personne;
            for (var k in personne) {
                var value = personne[k];
                inputHiddenPersonne.push(<input type={"hidden"} key={index + k}
                                                name={"personne[" + index + "][" + k + "]"} value={value}/>)
            }
        });
        setInputHiddenPersonne(inputHiddenPersonne);
        setPersonnes({...listPers});
    }

    useEffect(() => {
        const listPers = {};
        buildListPersonnes(listPers);
    }, [listAssure]);

    const onSubmit = (values) => {
        values["personne--date-naiss"] = values["personne--date-naiss"].format("YYYY-MM-DD");
        if (dataAssure.isEdit) {
            const assure = {...dataAssure.assure};
            if (typeof values["personne--nationalite"] == "object")
                assure.assurePrincipal["personne--nationalite"].alpha2code = values["personne--nationalite"].alpha2code;
            else assure.assurePrincipal["personne--nationalite"].alpha2code = values["personne--nationalite"];
            values["personne--nationalite"] = {...assure.assurePrincipal["personne--nationalite"]};
            if (index > -1) {
                listAssure[index] = {
                    assurePrincipal: {
                        'personne--num_client': listAssure[index].assurePrincipal['personne--num_client'],
                        ...values
                    },
                    conjoint: [],
                    enfants: []
                }

                setListAssure(listAssure);
            }
        } else {
            values["personne--nationalite"] = {
                alpha2code: values["personne--nationalite"],
                widget: "<span class=\"wpcf7-form-control-wrap personne--nationalite\"><select name=\"personne--nationalite\" class=\"d-flex w-100 swisscare-style\" id=\"personne--nationalite\" aria-required=\"true\" aria-invalid=\"false\" required=\"required\"><option><\/option><option value=\"AF\">Afghanistan<\/option><option value=\"AX\">\u00c5land<\/option><option value=\"AL\">Albanie<\/option><option value=\"DZ\">Alg\u00e9rie<\/option><option value=\"AS\">Samoa am\u00e9ricaines<\/option><option value=\"AD\">Andorre<\/option><option value=\"AO\">Angola<\/option><option value=\"AI\">Anguilla<\/option><option value=\"AQ\">Antarctique<\/option><option value=\"AG\">Antigua-et-Barbuda<\/option><option value=\"AR\">Argentine<\/option><option value=\"AM\">Arm\u00e9nie<\/option><option value=\"AW\">Aruba<\/option><option value=\"AU\">Australie<\/option><option value=\"AZ\">Azerba\u00efdjan<\/option><option value=\"BS\">Bahamas<\/option><option value=\"BH\">Bahre\u00efn<\/option><option value=\"BD\">Bangladesh<\/option><option value=\"BB\">Barbade<\/option><option value=\"BY\">Bi\u00e9lorussie<\/option><option value=\"BZ\">Belize<\/option><option value=\"BJ\">B\u00e9nin<\/option><option value=\"BM\">Bermudes<\/option><option value=\"BT\">Bhoutan<\/option><option value=\"BO\">Bolivie<\/option><option value=\"BQ\">Bonaire, Saint-Eustache et Saba<\/option><option value=\"BA\">Bosnie-Herz\u00e9govine<\/option><option value=\"BW\">Botswana<\/option><option value=\"BV\">\u00cele Bouvet<\/option><option value=\"BR\">Br\u00e9sil<\/option><option value=\"IO\">Territoire britannique de l&#039;oc\u00e9an Indien<\/option><option value=\"BN\">Brunei<\/option><option value=\"BG\">Bulgarie<\/option><option value=\"BF\">Burkina Faso<\/option><option value=\"BI\">Burundi<\/option><option value=\"CV\">Cap Vert<\/option><option value=\"KH\">Cambodge<\/option><option value=\"CM\">Cameroun<\/option><option value=\"CA\">Canada<\/option><option value=\"KY\">\u00celes Ca\u00efmans<\/option><option value=\"CF\">R\u00e9publique centrafricaine<\/option><option value=\"TD\">Tchad<\/option><option value=\"CL\">Chili<\/option><option value=\"CN\">Chine<\/option><option value=\"CX\">\u00cele Christmas<\/option><option value=\"CC\">\u00celes Cocos<\/option><option value=\"CO\">Colombie<\/option><option value=\"KM\">Comores<\/option><option value=\"CG\">Congo<\/option><option value=\"CD\">Congo (R\u00e9p. d\u00e9m.)<\/option><option value=\"CK\">\u00celes Cook<\/option><option value=\"CR\">Costa Rica<\/option><option value=\"CI\">C\u00f4te d&#039;Ivoire<\/option><option value=\"HR\">Croatie<\/option><option value=\"CU\">Cuba<\/option><option value=\"CW\">Cura\u00e7ao<\/option><option value=\"CY\">Chypre<\/option><option value=\"DJ\">Djibouti<\/option><option value=\"DM\">Dominique<\/option><option value=\"DO\">R\u00e9publique dominicaine<\/option><option value=\"EC\">\u00c9quateur<\/option><option value=\"EG\">\u00c9gypte<\/option><option value=\"SV\">Salvador<\/option><option value=\"GQ\">Guin\u00e9e-\u00c9quatoriale<\/option><option value=\"ER\">\u00c9rythr\u00e9e<\/option><option value=\"ET\">\u00c9thiopie<\/option><option value=\"FK\">\u00celes Malouines<\/option><option value=\"FO\">\u00celes F\u00e9ro\u00e9<\/option><option value=\"FJ\">Fidji<\/option><option value=\"GF\">Guayane<\/option><option value=\"TF\">Terres australes et antarctiques fran\u00e7aises<\/option><option value=\"GA\">Gabon<\/option><option value=\"GM\">Gambie<\/option><option value=\"GE\">G\u00e9orgie<\/option><option value=\"GH\">Ghana<\/option><option value=\"GI\">Gibraltar<\/option><option value=\"GL\">Groenland<\/option><option value=\"GD\">Grenade<\/option><option value=\"GP\">Guadeloupe<\/option><option value=\"GU\">Guam<\/option><option value=\"GT\">Guatemala<\/option><option value=\"GG\">Ile de Guernesey<\/option><option value=\"GN\">Guin\u00e9e<\/option><option value=\"GW\">Guin\u00e9e-Bissau<\/option><option value=\"GY\">Guyane<\/option><option value=\"HT\">Ha\u00efti<\/option><option value=\"HM\">\u00celes Heard-et-MacDonald<\/option><option value=\"VA\">voir Saint<\/option><option value=\"HN\">Honduras<\/option><option value=\"HK\">Hong Kong<\/option><option value=\"IN\">Inde<\/option><option value=\"ID\">Indon\u00e9sie<\/option><option value=\"IR\">Iran<\/option><option value=\"IQ\">Irak<\/option><option value=\"IM\">\u00cele de Man<\/option><option value=\"IL\">Isra\u00ebl<\/option><option value=\"JM\">Jama\u00efque<\/option><option value=\"JP\">Japon<\/option><option value=\"JE\">Jersey<\/option><option value=\"JO\">Jordanie<\/option><option value=\"KZ\">Kazakhstan<\/option><option value=\"KE\">Kenya<\/option><option value=\"KI\">Kiribati<\/option><option value=\"KP\">Cor\u00e9e du Nord<\/option><option value=\"KR\">Cor\u00e9e du Sud<\/option><option value=\"KW\">Kowe\u00eft<\/option><option value=\"KG\">Kirghizistan<\/option><option value=\"LA\">Laos<\/option><option value=\"LB\">Liban<\/option><option value=\"LS\">Lesotho<\/option><option value=\"LR\">Liberia<\/option><option value=\"LY\">Libye<\/option><option value=\"LI\">Liechtenstein<\/option><option value=\"MO\">Macao<\/option><option value=\"MK\">Mac\u00e9doine<\/option><option value=\"MG\" selected=\"selected\">Madagascar<\/option><option value=\"MW\">Malawi<\/option><option value=\"MY\">Malaisie<\/option><option value=\"MV\">Maldives<\/option><option value=\"ML\">Mali<\/option><option value=\"MH\">\u00celes Marshall<\/option><option value=\"MQ\">Martinique<\/option><option value=\"MR\">Mauritanie<\/option><option value=\"MU\">\u00cele Maurice<\/option><option value=\"YT\">Mayotte<\/option><option value=\"MX\">Mexique<\/option><option value=\"FM\">Micron\u00e9sie<\/option><option value=\"MD\">Moldavie<\/option><option value=\"MC\">Monaco<\/option><option value=\"MN\">Mongolie<\/option><option value=\"ME\">Mont\u00e9n\u00e9gro<\/option><option value=\"MS\">Montserrat<\/option><option value=\"MA\">Maroc<\/option><option value=\"MZ\">Mozambique<\/option><option value=\"MM\">Myanmar<\/option><option value=\"NA\">Namibie<\/option><option value=\"NR\">Nauru<\/option><option value=\"NP\">N\u00e9pal<\/option><option value=\"NC\">Nouvelle-Cal\u00e9donie<\/option><option value=\"NZ\">Nouvelle-Z\u00e9lande<\/option><option value=\"NI\">Nicaragua<\/option><option value=\"NE\">Niger<\/option><option value=\"NG\">Nig\u00e9ria<\/option><option value=\"NU\">Niue<\/option><option value=\"NF\">\u00cele de Norfolk<\/option><option value=\"MP\">\u00celes Mariannes du Nord<\/option><option value=\"NO\">Norv\u00e8ge<\/option><option value=\"OM\">Oman<\/option><option value=\"PK\">Pakistan<\/option><option value=\"PW\">Palaos<\/option><option value=\"PS\">Palestine<\/option><option value=\"PA\">Panama<\/option><option value=\"PG\">Papouasie-Nouvelle-Guin\u00e9e<\/option><option value=\"PY\">Paraguay<\/option><option value=\"PE\">P\u00e9rou<\/option><option value=\"PH\">Philippines<\/option><option value=\"PN\">\u00celes Pitcairn<\/option><option value=\"PR\">Porto Rico<\/option><option value=\"QA\">Qatar<\/option><option value=\"XK\">Kosovo<\/option><option value=\"RE\">La R\u00e9union<\/option><option value=\"RO\">Roumanie<\/option><option value=\"RU\">Russie<\/option><option value=\"RW\">Rwanda<\/option><option value=\"BL\">Saint-Barth\u00e9lemy<\/option><option value=\"SH\">Sainte-H\u00e9l\u00e8ne<\/option><option value=\"KN\">Saint-Christophe-et-Ni\u00e9v\u00e8s<\/option><option value=\"LC\">Saint-Lucie<\/option><option value=\"MF\">Saint-Martin<\/option><option value=\"PM\">Saint-Pierre-et-Miquelon<\/option><option value=\"VC\">Saint-Vincent-et-les-Grenadines<\/option><option value=\"WS\">Samoa<\/option><option value=\"SM\">Saint-Marin<\/option><option value=\"ST\">Sao Tom\u00e9-et-Principe<\/option><option value=\"SA\">Arabie Saoudite<\/option><option value=\"SN\">S\u00e9n\u00e9gal<\/option><option value=\"RS\">Serbie<\/option><option value=\"SC\">Seychelles<\/option><option value=\"SL\">Sierra Leone<\/option><option value=\"SG\">Singapour<\/option><option value=\"SX\">Saint Martin (partie n\u00e9erlandaise)<\/option><option value=\"SB\">\u00celes Salomon<\/option><option value=\"SO\">Somalie<\/option><option value=\"ZA\">Afrique du Sud<\/option><option value=\"GS\">G\u00e9orgie du Sud-et-les \u00celes Sandwich du Sud<\/option><option value=\"SS\">Soudan du Sud<\/option><option value=\"LK\">Sri Lanka<\/option><option value=\"SD\">Soudan<\/option><option value=\"SR\">Surinam<\/option><option value=\"SJ\">Svalbard et Jan Mayen<\/option><option value=\"SZ\">Swaziland<\/option><option value=\"SY\">Syrie<\/option><option value=\"TW\">Ta\u00efwan<\/option><option value=\"TJ\">Tadjikistan<\/option><option value=\"TZ\">Tanzanie<\/option><option value=\"TH\">Tha\u00eflande<\/option><option value=\"TL\">Timor oriental<\/option><option value=\"TG\">Togo<\/option><option value=\"TK\">Tokelau<\/option><option value=\"TO\">Tonga<\/option><option value=\"TT\">Trinit\u00e9 et Tobago<\/option><option value=\"TN\">Tunisie<\/option><option value=\"TR\">Turquie<\/option><option value=\"TM\">Turkm\u00e9nistan<\/option><option value=\"TC\">\u00celes Turques-et-Ca\u00efques<\/option><option value=\"TV\">Tuvalu<\/option><option value=\"UG\">Uganda<\/option><option value=\"UA\">Ukraine<\/option><option value=\"AE\">\u00c9mirats arabes unis<\/option><option value=\"GB\">Royaume-Uni (UK)<\/option><option value=\"UM\">\u00celes mineures \u00e9loign\u00e9es des \u00c9tats-Unis<\/option><option value=\"US\">\u00c9tats-Unis (USA)<\/option><option value=\"UY\">Uruguay<\/option><option value=\"UZ\">Ouzb\u00e9kistan<\/option><option value=\"VU\">Vanuatu<\/option><option value=\"VE\">Venezuela<\/option><option value=\"VN\">Vi\u00eat Nam<\/option><option value=\"VG\">\u00celes Vierges britanniques<\/option><option value=\"VI\">\u00celes Vierges des \u00c9tats-Unis<\/option><option value=\"WF\">Wallis-et-Futuna<\/option><option value=\"EH\">Sahara Occidental<\/option><option value=\"YE\">Y\u00e9men<\/option><option value=\"ZM\">Zambie<\/option><option value=\"ZW\">Zimbabwe<\/option><\/select><\/span>"
            }
            listAssure[`${Object.keys(listAssure).length + 1}`] = {
                assurePrincipal: {
                    'personne--num_client': '',
                    ...values
                },
                conjoints: [],
                enfants: []
            }
        }
        setListAssure(listAssure);
        const listPers = {};
        buildListPersonnes(listPers);
        onUpdateAssuresList({...listPers});
        setShowModal(!showModal);
    }

    const onCancel = () => {
        setShowModal(!showModal);
    }

    const handleEdit = (index, assure) => {
        setIndex(index);
        const assure1 = {...assure};
        setDataAssure({
            isEdit: true,
            assure: assure1
        });
        setShowModal(!showModal);
    }

    const handleDelete = (index) => {
        if (index > -1) {
            delete listAssure[index];
            listAssure = {...listAssure};
            setListAssure(listAssure);
            const listPers = {};
            buildListPersonnes(listPers);
            onUpdateAssuresList({...listPers})
        }
    }

    const onAjoutAssure = (e) => {
        setDataAssure({
            isEdit: false,
            assure: {}
        });
        form.resetFields();
        setShowModal(!showModal);
    };

    const onChangeInFamille = () => {

    }

    const panels = [];
    Object.entries(listAssure).map( (assures, key) => panels.push(key))

    return (
        <CCard>
            <CModal
                onCancel={onCancel}
                destroyOnClose={true}
                title={
                    dataAssure.isEdit
                        ? `${dataAssure.assure.assurePrincipal['personne--prenom']} ${dataAssure.assure.assurePrincipal['personne--nom']}`
                        : `${ctx.ajouter_assure}`
                }
                visible={showModal}
                footer={[]}>
                <FormAssure
                    dataAssure={dataAssure}
                    form={form}
                    onSubmit={onSubmit}/>
            </CModal>

            <CCollapse
                defaultActiveKey={panels}
                bordered={false}
                ghost={false}>
                {Object.entries(listAssure).map( (assures, key) => {
                    const index = assures[0];
                    const assure = assures[1];
                    let datenaiss = assure.assurePrincipal["personne--date-naiss"];
                    var mDateNaiss = moment(datenaiss);
                    if (moment.isMoment(datenaiss)) datenaiss = datenaiss.format("DD/MM/YYYY");
                    if (mDateNaiss.isValid()) mDateNaiss.format("DD/MM/YYYY");
                    if (!mDateNaiss.isValid()) {
                        mDateNaiss = moment(datenaiss, "DD/MM/YYYY");
                        datenaiss = mDateNaiss.format("DD/MM/YYYY");
                    }
                    var age = moment().diff(mDateNaiss,"y");
                    return (
                        <Panel
                            showArrow={false}
                            key={key}
                            header={`${assure.assurePrincipal['personne--prenom']} ${assure.assurePrincipal['personne--nom']}`}>
                            <div>
                                <CRow justify='center'>
                                    <Col xs={24} sm={11}>
                                        <p className="label">{ctx.prenom}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p>{assure.assurePrincipal['personne--prenom']}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p className="label">{ctx.nom}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p>{assure.assurePrincipal['personne--nom']}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p className="label">{ctx.date_naissance}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        {datenaiss}
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p className="label">{ctx.passeport}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p>{assure.assurePrincipal['personne--passeport']}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p className="label">{ctx.nationalite}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        {assure.assurePrincipal['personne--nationalite']?.countryName}
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p className="label">{ctx.age}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p>{age}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p className="label">{ctx.devis_pp}</p>
                                    </Col>
                                    <Col xs={24} sm={11}>
                                        <p>{assure.assurePrincipal['personne--devis-pp']} {ctx.currency}</p>
                                    </Col>
                                </CRow>
                                <DivFlex justify="flex-end">
                                    <div>
                                        <CButton
                                            onClick={() => handleEdit(index, assure)}>
                                            {ctx.modifier_assure}
                                        </CButton>
                                    </div>
                                    <div>
                                        <Popconfirm
                                            placement="topRight"
                                            title={ctx.confirm_text}
                                            onConfirm={() => handleDelete(index)}
                                            okText={ctx.oui}
                                            cancelText={ctx.non}
                                        >
                                            <CButton
                                                color='primary'>
                                                {ctx.remove_assure}
                                            </CButton>
                                        </Popconfirm>
                                    </div>
                                </DivFlex>
                            </div>
                            {options.formuleChoisi.widgetAssure !== 'individuel' && (
                                <ConfigProvider locale={frLocale}>
                                    <Personne options={options} personne={assure.assurePrincipal} indice={index} onChangeInFamille={onChangeInFamille} />
                                </ConfigProvider>
                            )}
                        </Panel>
                    )
                })}
                {inputHiddenPersonne}
            </CCollapse>

            <DivFlex justify='center'>
                <CButton
                    color='primary'
                    onClick={onAjoutAssure}>
                    {ctx.ajouter_assure}
                </CButton>
            </DivFlex>

        </CCard>
    )
};

export default AssureInfo;