import React, {useContext, useEffect} from 'react';

import {Col, Form} from 'antd';
import CButton from "../components/CButton";
import CInput from "../components/CInput";
import CRow from "../components/CRow";
import LabelContext from "../context";
import DivFlex from "../components/DivFlex";
import CDate from "../components/CDate";
import moment from "moment";
import CNationalite, {DivStyled} from "../components/CNationalite";
import EditNationalite from "../components/EditNationalite";

const FormAssure = (props) => {
  const {
    form,
    dataAssure,
    onSubmit,
    onError
  } = props;

  const ctx = useContext(LabelContext);

  var initialValues = dataAssure.isEdit ? dataAssure.assure.assurePrincipal : {};
  if (!dataAssure.isEdit) {
    initialValues["personne--date-naiss"] = moment();
  } else {
    const initialValue = initialValues["personne--date-naiss"];
    initialValues["personne--date-naiss"] = moment(initialValue);
    if (!initialValues["personne--date-naiss"].isValid()) {
      initialValues["personne--date-naiss"] = moment(initialValue, "DD/MM/YYYY");
    }
  }
  const dateFormat = 'DD/MM/YYYY';
  const disabledDate = (current) => current && current > moment().startOf("day");

  useEffect(() => {
    form.resetFields();
  }, [initialValues]);

  const onNationaliteChange = React.useCallback((e) => {
    form.setFieldsValue({
      "personne--nationalite": e.target.value
    })
  },[]);
  return (
    <Form
      initialValues={initialValues}
      onFinish={onSubmit}
      name={initialValues.nom}
      form={form}
      onFinishFailed={onError}>
      <CRow>
        <Col md={11} xs={24}>
          <b>{ctx.prenom}</b>
          <Form.Item name='personne--prenom'>
            <CInput name='personne--prenom' placeholder={ctx.prenom}/>
          </Form.Item>
          <b>{ctx.nom}</b>
          <Form.Item name='personne--nom'>
            <CInput name='personne--nom' placeholder={ctx.nom}/>
          </Form.Item>
          <b>{ctx.date_naissance}</b>
          <Form.Item name="personne--date-naiss">
            <CDate name='personne--date-naiss' format={dateFormat} disabledDate={disabledDate}/>
          </Form.Item>
        </Col>
        <Col md={11} xs={24}>
          <b>{ctx.passeport}</b>
          <Form.Item name='personne--passeport'>
            <CInput name='personne--passeport' placeholder={ctx.passeport}/>
          </Form.Item>
          <b>{ctx.nationalite}</b>
          <Form.Item name='personne--nationalite'>
            {dataAssure.isEdit &&
              <EditNationalite name={'personne--nationalite'} onChange={onNationaliteChange} html={initialValues['personne--nationalite']?.widget} value={initialValues['personne--nationalite']?.alpha2code}/>
            }
            {!dataAssure.isEdit &&
              <CNationalite name='personne--nationalite' onChange={onNationaliteChange}/>
            }
          </Form.Item>
        </Col>
      </CRow>
      <DivFlex justify='flex-end'>
        <CButton form={form} color="primary" type="submit">
          Valider
        </CButton>
      </DivFlex>
    </Form>
  )
};

export default FormAssure;