<?php
add_action('init', 'v2_initial_action', 10, 0);
add_action('admin_menu', "v2_plugin_menu");
add_action('enqueue_mc_scripts', "v2_enqueue_mc_scripts");
add_action('wp_enqueue_scripts', "v2_enqueue_scripts");

function v2_initial_action()
{
    add_shortcode(MC_V2_WORKFLOW_SHORTCODE_NAME, 'MCV2_register_workflow_shortcode');
    add_shortcode(MC_V2_ASSISTANT_SHORTCODE_NAME, 'MCV2_register_assistant_shortcode');
}

function MCV2_register_workflow_shortcode($atts, $content, $shortcode_tag)
{
    $parent_source = get_option('parent_source');
    $token_type = get_option('token_type');
    $access_token = get_option('access_token');

    loadReactAssets('/assets/famille');

    $post_id = $atts['id'];

    ob_start();
    if ($parent_source != false && $token_type != false && $access_token != false) {
        $url = add_query_arg($_GET, $parent_source . '/api/v1/obtenir_produit/' . $post_id);
        $request = Requests::get($url, [
            'Authorization' => $token_type . ' ' . $access_token
        ]);
        if ($request->status_code === 200) {
            $results = json_decode($request->body, true);
            $forms = $results["forms"];
            $screenWidth = $results["screenWidth"];
            $background = $results["background"];
            ?>
            <div class="bg-filter"></div>
            <div class="form-wrapper" style="max-width: 1200px">
                <!-- Circles which indicates the steps of the form: -->
                <div class="left-side" style="display: none" data-form-id="<?= $post_id ?>">
                    <?php
                    foreach ($forms as $key => $etape) {
                        ?>
                        <div class="img-content">
                            <img class="avatar" src=""/>
                        </div>
                        <div class="talkbubble">
                            <p class="didacticiel"></p>
                        </div>
                        <?php
                    }
                    ?>
                </div>
                <div class="form-container">
                    <?php
                    if (!empty($forms)) {
                        foreach ($forms as $etape) {
                            ?>
                            <div class="tab">
                                <?= $etape["content"] ?>
                            </div>
                            <?php
                        }
                    }
                    ?>
                </div>
            </div>
            <style>
                .bg-filter {
                    border-radius: 8px;
                    display: inline-block;
                    z-index: -20;
                    line-height: 0;
                    opacity: 1;
                    background-color: rgb(249, 253, 253);
                    background-image: url('<?= $background ?>');
                    background-position: center top;
                    background-size: cover;
                    height: 100%;
                    width: 100%;
                    position: absolute;
                    top: 0;
                    left: 0;
                    max-width: unset !important;
                    margin-left: unset !important;
                    margin-right: unset !important;
                }

                .bg-filter::before {
                    content: "";
                    position: absolute;
                    width: 100%;
                    height: 100%;
                    pointer-events: none;
                    z-index: 1;
                    opacity: 0.81;
                    background-color: rgb(255, 255, 255);
                }
                .form-wrapper {
                    max-width: <?= $screenWidth != null ? $screenWidth : '786' ?>px;
                }

                .form-container {
                    margin-top: 15px;
                    background: unset !important;
                    box-shadow: unset !important;
                }
            </style>
            <?php
        } else {
            print(__('Erreur lors de l\'obtention de la liste des formulaires du produit', 'mcv2'));
        }
    }
    $html = ob_get_contents();
    ob_end_clean();
    enqueue_shortcode_assets();
    return $html;
}

function MCV2_register_assistant_shortcode($atts, $content, $shortcode_tag)
{
//    dspv2_loadReactAssets('/assets/famille');

    $typeformId = $atts['id'];
    $redirection = $atts['page_redirection'];
    $parent_source = get_option('parent_source');
    $token_type = get_option('token_type');
    $access_token = get_option('access_token');
    $codeApporteur = "";
    if (!in_array(false, compact('parent_source', 'token_type', 'access_token'))) {
        $request = Requests::get($parent_source . '/api/v2/obtenir_code_apporteur', [
            'Authorization' => $token_type . ' ' . $access_token
        ]);
        if ($request->status_code === 200) {
            $codeApporteur = json_decode($request->body);
            if (!is_numeric($codeApporteur)) {
                throw new Exception("Code apporteur non trouvé");
            }
        } else {
            throw new Exception("Code apporteur non trouvé");
        }
    }
    ob_start();
    ?>
    <style>
        .tf-v1-widget iframe {
            border-radius: unset !important;
        }
    </style>
    <!-- Bug visuel si on enlève ceci -->
    <div data-tf-widget="<?= $typeformId ?>" style="display: none"></div>
    <div id="form1" style="display: flex; align-items: stretch; height: calc(100vh - 129px);"></div>
    <!-- <div data-tf-widget="{!! $form_id !!}" data-tf-hide-headers data-tf-hide-footer data-tf-opacity="0" id="form" data-tf-height="600"></div> -->
    <script src="//embed.typeform.com/next/embed.js"></script>
    <script>
        const {refresh, unmount} = window.tf.createWidget('<?= $typeformId ?>', {
            container: document.querySelector("#form1"),
            hideHeaders: true,
            hideFooter: true,
            inlineOnMobile: true,
            hidden: {
                source: window.location.href,
                typeformid: "<?= $typeformId ?>",
                page_redirection: "<?=$redirection ?>",
                apporteur: "<?= $codeApporteur ?>"
            },
            onSubmit: submit
        });

        function submit(event) {

        }
    </script>
    <?php
    $html = ob_get_contents();
    ob_end_clean();
    enqueue_shortcode_assets();
    return $html;
}

function v2_plugin_menu()
{
    $str = MENU_PRODUIT_MONDIALCARE_V2;
    $parent_slug = 'mc_contrats';
    $hook = add_submenu_page(
        $parent_slug,
        $str,
        $str,
        'manage_options',
        'v2_mondialcare_product',
        "produits_v2"
    );

    add_action("load-$hook", 'v2_screen_option');
}

function v2_screen_option()
{
    global $tableV2;

    $tableV2 = new ProductMCTableV2();
}

function produits_v2()
{
    global $tableV2;
    ?>
    <style>
        .column-nativeLang {
            width: 150px !important;
        }

        .grid {
            display: flex;
            flex-direction: row;
        }

        .grid .grid-item {
            width: 100%;
            padding-top: calc(4px * 2);
            padding-bottom: calc(4px * 2);
        }

        .t-right {
            text-align: right;
        }

        .texte-rouge {
            color: red;
        }
    </style>
    <?= mondialcare_header(); ?>
    <div class="wrap">
        <div class="grid">
            <div class="grid-item">
                <h2>Liste des produits</h2>
            </div>
        </div>
        <p>
        <h5>Etapes d'intégrations :</h5>
        <ol>
            <li>Créez une page pour intégrer le gestionnaire du workflow.</li>
            <li>Collez-y le shortcode correspondant à la langue souhaitée.<br/>
                ex: [workflow_mc id='738523']
            </li>
            <li>Publiez la page et recupérez le permalien</li>
            <li>Dans le tableau ci dessous, mettez à jour la valeur du shortcode assistant_mc.
                Assurez vous que la valeur du paramètre page_redirection est égale au permalien récupéré précédement.
                <br/>
                ex: [assistant_mc id='gw5lIHPf' page_redirection='https://monsite.com/assistant-en-francais/']
            </li>
        </ol>
        </p>
        <div id="poststuff">
            <div id="post-body" class="metabox-holder columns-2">
                <div id="post-body-content">
                    <div class="meta-box-sortables ui-sortable">
                        <form method="post">
                            <?php
                            $tableV2->prepare_items();
                            $tableV2->display();
                            ?>
                        </form>
                    </div>
                </div>
            </div>
            <br class="clear">
        </div>
    </div>
    <?= mondialcare_footer() ?>
    <?php
}

function v2_enqueue_scripts()
{
    wp_register_script('generatecardproduit', plugins_url('/assets/public/js/generatecardproduit/index.js', __FILE__), [], false, true);
    wp_register_script('boucleAssurer', plugins_url('/assets/public/js/boucleAssurer/index.js', __FILE__), [], false, true);
    wp_register_script('datespicker', plugins_url('/assets/public/js/dates/app.js', __FILE__), [], false, true);
    wp_register_script('inputTelInput.js', plugins_url('/assets/inputTelephone/js/intlTelInput.js', __FILE__), [], false, true);
    wp_register_script('utils.js', plugins_url('/assets/inputTelephone/js/utils.js', __FILE__), [], false, true);
    wp_register_script('load-data.js', plugins_url('/assets/load-data.js', __FILE__), [], false, true);

    wp_register_style('inputTelInput.css', plugins_url('/assets/inputTelephone/css/intlTelInput.css', __FILE__), [], false);
    wp_register_style('recap.css', plugins_url('/assets/css/recap.css', __FILE__), [], false);
}

function v2_enqueue_mc_scripts()
{
    wp_enqueue_script('generatecardproduit');
    wp_enqueue_script('boucleAssurer');
    wp_enqueue_script('datespicker');
    wp_enqueue_script('inputTelInput.js');
    wp_enqueue_script('utils.js');
    wp_enqueue_script('load-data.js');

    wp_enqueue_style('inputTelInput.css');
    wp_enqueue_style('recap.css');
}