<?php
if (!class_exists('WP_List_Table')) {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

class ProductMCTableV2 extends WP_List_Table
{
    /** Class constructor */
    public function __construct()
    {

        parent::__construct([
            'singular' => __('Product', 'mcv2'), //singular name of the listed records
            'plural' => __('Products', 'mcv2'), //plural name of the listed records
            'ajax' => false //does this table support ajax?
        ]);
    }


    /**
     * Retrieve Product data from the database
     *
     * @param int $per_page
     * @param int $page_number
     *
     * @return mixed
     */
    public static function get_product($per_page = 50, $page_number = 1)
    {
        $array_resultat = [];
        $parent_source = get_option('parent_source');
        $token_type = get_option('token_type');
        $access_token = get_option('access_token');
        if (!in_array(false, compact('parent_source', 'token_type', 'access_token'))) {
            $request = Requests::get($parent_source . '/api/v2/obtenir_typeforms', [
                'Authorization' => $token_type . ' ' . $access_token
            ]);
            $all_product = [];
            if ($request->status_code === 200) {
                $all_product = json_decode($request->body);
            }
            $redir_url = get_option(MC_V2_REDIR_URL_OPTION_NAME, "");
            foreach ($all_product as $key => $res) {
                $array_resultat[$key] = (array)$res;
                $array_resultat[$key]["shortcode"] = [
                    [
                        "shortcode" => "[".MC_V2_ASSISTANT_SHORTCODE_NAME." id='" . $res->typeformId . "' page_redirection='" . $redir_url . "']",
                        "readonly"  => false
                    ],
                    [
                        "shortcode" => "[".MC_V2_WORKFLOW_SHORTCODE_NAME." id='" . $res->productId . "']",
                        "readonly"  => true
                    ]
                ];
            }
        }
        return $array_resultat;
    }


    /**
     * Returns the count of records in the database.
     *
     * @return null|string
     */
    public static function record_count()
    {
        $array_resultat = [];
        $parent_source = get_option('parent_source');
        $token_type = get_option('token_type');
        $access_token = get_option('access_token');

        if ($parent_source != false && $token_type != false && $access_token != false) {
            $request = Requests::get($parent_source . '/api/v2/obtenir_typeforms', [
                'Authorization' => $token_type . ' ' . $access_token
            ]);
            if ($request->status_code === 200)
                $array_resultat = json_decode($request->body);

        }
        return count($array_resultat);
    }


    /** Text displayed when no customer data is available */
    public function no_items()
    {
        _e('Pas de produit disponible pour ce site, Veuillez paramétrer le plugin', 'mcv2');
    }


    /**
     * Render a column when no column specific method exist.
     *
     * @param array $item
     * @param string $column_name
     *
     * @return mixed
     */
    public function column_default($item, $column_name)
    {
        switch ($column_name) {
            case 'shortcode':
            case 'nativeLang':
                return "<svg width=\"30\" height=\"20\"><image href='" . $item["flag"] . "' width=\"30\"
                                           height=\"20\"/></svg> &nbsp; $item[$column_name]";
            default:
                return print_r($item, true); //Show the whole array for troubleshooting purposes
        }
    }

    /**
     * Method for shortcode column
     *
     * @param array $item an array of DB data
     *
     * @return string
     */
    function column_shortcode($item)
    {
        $output = "";
        foreach ($item['shortcode'] as $shortcode) {
            $output .= '<span class="shortcode"><input style="margin-bottom: 3px;" type="text"'
//                . ' onfocus="this.select();" '
                . ($shortcode["readonly"] ? 'readonly="readonly"' : "")
                . ' value="' . esc_attr($shortcode["shortcode"]) . '"'
                . ' class="large-text code" /></span>';
        }

        return $output;
    }


    /**
     *  Associative array of columns
     *
     * @return array
     */
    function get_columns()
    {
        $columns = [
//            'typeformId' => __('ID', 'mcv2'),
//            'lang' => __('Drapeaux', 'mcv2'),
            'shortcode' => __('Shortcodes', 'mcv2'),
            'nativeLang' => __('Langue', 'mcv2'),
        ];

        return $columns;
    }


    /**
     * Columns to make sortable.
     *
     * @return array
     */
    public function get_sortable_columns()
    {
        $sortable_columns = array(
            'id' => array('id', true)
        );

        return $sortable_columns;
    }


    /**
     * Handles data query and filter, sorting, and pagination.
     */
    public function prepare_items()
    {

        $this->_column_headers = $this->get_column_info();

        /** Process bulk action */
        $this->process_bulk_action();

        $per_page = 50;
        $current_page = $this->get_pagenum();
        $total_items = self::record_count();


        $this->set_pagination_args([
            'total_items' => $total_items, //WE have to calculate the total number of items
            'per_page' => $per_page //WE have to determine how many items to show on a page
        ]);

        $this->items = self::get_product($per_page, $current_page);
    }
}