<?php

/**
Plugin Name: MondialCare Form
Plugin URI:
Description: MondialCare Plugin
Version: 1.0.7
Author: MondialCare
Author URI: https://www.mondialcare.eu
 */
include __DIR__ . '/ProductTable.php';
define('MC_PLUGIN_VERSION', '1.0.7');
function my_plugin_configure_ssl() {
    // configure SSL for all requests
    $cert_path = plugin_dir_path( __FILE__ ) . 'ca-bundle.crt';
	//set Requests certificate path
	Requests::set_certificate_path($cert_path);
}

add_action( 'init', 'my_plugin_configure_ssl' );

add_filter('site_transient_update_plugins', 'plugin_update_checker');
// show message on admin notice hook
add_action('admin_notices', 'plugin_update_message');
// filter  plugins_api hook to get plugin info
add_filter('plugins_api', 'plugin_info', 10, 3);

function plugin_update_message() {
	// check if the transient is set, and display the update message
	if ( get_transient( 'mc_update_message' ) ) {
		echo '<div class="updated"><p>There is a new version of MondialCare Plugin available. <a href="' . admin_url( 'plugins.php' ) . '">Update now</a></p></div>';
		delete_transient( 'mc_update_message' );
	}
}

function plugin_update_checker($transient) {
	if ( empty( $transient->checked ) ) {
		return $transient;
	}

	$remote = wp_remote_get( 
		'http://cdn.mondialcare.eu/info.json',
		array(
			'timeout' => 10,
			'headers' => array(
				'Accept' => 'application/json'
			)
		)
	);

	if( 
		is_wp_error( $remote )
		|| 200 !== wp_remote_retrieve_response_code( $remote )
		|| empty( wp_remote_retrieve_body( $remote ) )
	) {
		return $transient;	
	}
	
	$remote = json_decode( wp_remote_retrieve_body( $remote ) );
	if( 
		$remote 
		&& version_compare( $remote->version, MC_PLUGIN_VERSION, '>' )
		&& version_compare( $remote->requires, get_bloginfo( 'version' ), '<=' )
		&& version_compare( $remote->requires_php, PHP_VERSION, '<' )
	) {
		$res = new stdClass();
		$res->slug = $remote->slug;
		$res->plugin = plugin_basename( __FILE__ );
		$res->new_version = $remote->version;
		$res->tested = $remote->tested;
		$res->package = $remote->download_link;
		$transient->response[$res->plugin] = $res;
		set_transient("mc_update_message", "1", 3600);
	}
	return $transient;
}

function plugin_info($res, $action, $args) {
	// do nothing if this is not about getting plugin information
	if( 'plugin_information' !== $action ) {
		return $res;
	}

	// do nothing if it is not our plugin
	if( plugin_basename( __DIR__ ) !== $args->slug ) {
		return $res;
	}

	$remote = wp_remote_get( 
		'http://cdn.mondialcare.eu/info.json',
		array(
			'timeout' => 10,
			'headers' => array(
				'Accept' => 'application/json'
			)
		)
	);

	if( 
		is_wp_error( $remote )
		|| 200 !== wp_remote_retrieve_response_code( $remote )
		|| empty( wp_remote_retrieve_body( $remote ) )
	) {
		return $res;	
	}

	$remote = json_decode( wp_remote_retrieve_body( $remote ) );
	
	$res = new stdClass();
	$res->name = $remote->name;
	$res->slug = $remote->slug;
	$res->author = $remote->author;
	$res->author_profile = $remote->homepage;
	$res->version = $remote->version;
	$res->tested = $remote->tested;
	$res->requires = $remote->requires;
	$res->download_link = $remote->download_link;
	$res->sections = array(
		'description' => "MondialCare Plugin",
		'installation' => "",
		'changelog' => $remote->sections->changelog
		// you can add your custom sections (tabs) here
	);
	// in case you want the screenshots tab, use the following HTML format for its content:
	// <ol><li><a href="IMG_URL" target="_blank"><img src="IMG_URL" alt="CAPTION" /></a><p>CAPTION</p></li></ol>
	if( ! empty( $remote->sections->screenshots ) ) {
		$res->sections[ 'screenshots' ] = $remote->sections->screenshots;
	}
	if(!empty($res->banners)) {
		$res->banners = array(
			'low' => $remote->banners->low,
			'high' => $remote->banners->high
		);
	}
	return $res;
}

class MC_Plugin
{

	// class instance
	static $instance;

	// customer WP_List_Table object
	public $products;

	// class constructor
	public function __construct()
	{
		add_filter('set-screen-option', [__CLASS__, 'set_screen'], 10, 3);
		include_once __DIR__ . '/contrats.php';
		include_once __DIR__ . '/devis.php';
		include_once __DIR__ . '/apporteur.php';
		add_action('admin_menu', [$this, 'plugin_menu']);
		include_once __DIR__ . '/pdf.php';
		include_once __DIR__ . '/personnalisation.php';
		include_once __DIR__ . '/configuration.php';
		add_action('wp_enqueue_scripts', function () {
			$this->registerReactAssets('/assets/famille');
		});

        include_once __DIR__ . '/extensions/v2/autoload.php';
        include_once __DIR__ . '/extensions/backup/autoload.php';
	}
	public function registerReactAssets($assetsPath) {
		$manifestFileName = '/asset-manifest.json';
		$filePath = __DIR__ . $assetsPath . $manifestFileName;
		if( file_exists($filePath) ) {
			$json = file_get_contents( $filePath );
			$v = json_decode($json, true);
			$entryPoints = $v['entrypoints'];
			foreach ( $entryPoints as $entryPoint ) {
				$assetPath = plugins_url($assetsPath . '/'. $entryPoint, __FILE__);
				if ( false !== strpos( $entryPoint, '/js/' ) ) {
					wp_register_script($entryPoint, $assetPath);
				} else if ( false !== strpos( $entryPoint, '/css/' ) ) {
					wp_register_style($entryPoint, $assetPath);
				}
			}
		}
	}
	public static function set_screen($status, $option, $value)
	{
		return $value;
	}
	public function plugin_menu()
	{

		$hook = add_submenu_page(
			'mc_contrats',
			'Produits Mondialcare',
			'Produits MondialCare',
			'manage_options',
			'mondialcare_product',
			[$this, 'index']
		);

		add_action("load-$hook", [$this, 'screen_option']);
	}


	/**
	 * Plugin index page
	 */
	public function index()
	{
?>
		<style>
			.column-id {
				width: 70px !important;
			}

			.column-title {
				width: 250px !important;
			}

			.column-locale {
				width: 100px !important;
			}

			.grid {
				display: flex;
				flex-direction: row;
			}

			.grid .grid-item {
				width: 100%;
				padding-top: calc(4px * 2);
				padding-bottom: calc(4px * 2);
			}

			.t-right {
				text-align: right;
			}
		</style>
		<?= mondialcare_header(); ?>
		<div class="wrap">
			<div class="grid">
				<div class="grid-item">
					<h2>Liste des produits Mondialcare</h2>
				</div>
			</div>
			<div id="poststuff">
				<div id="post-body" class="metabox-holder columns-2">
					<div id="post-body-content">
						<div class="meta-box-sortables ui-sortable">
							<form method="post">
								<?php
								$this->products->prepare_items();
								$this->products->display();
								?>
							</form>
						</div>
					</div>
				</div>
				<br class="clear">
			</div>
		</div>
		<?= mondialcare_footer() ?>
	<?php
	}

	/**
	 * Screen options
	 */
	public function screen_option()
	{
		$this->products = new ProductTable();
	}


	/** Singleton instance */
	public static function get_instance()
	{
		if (!isset(self::$instance)) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}

/**
 * Create shortcode for MondialCare product
 */
add_action('init', 'initial_action', 10, 0);
function initial_action()
{
    add_action("wp_enqueue_scripts", function() {
        wp_enqueue_script('jquery', false, array(), false, false);
    });
	add_shortcode('mc_produit', 'register_shortcode');

}

function loadReactAssets($assetsPath) {
	$manifestFileName = '/asset-manifest.json';
	$filePath = __DIR__ . $assetsPath . $manifestFileName;
	if( file_exists($filePath) ) {
		$json = file_get_contents( $filePath );
		$v = json_decode($json, true);
		$entryPoints = $v['entrypoints'];
		foreach ( $entryPoints as $entryPoint ) {
			$assetPath = plugins_url($assetsPath . '/'. $entryPoint, __FILE__);
			if ( false !== strpos( $entryPoint, '/js/' ) ) {
				wp_enqueue_script($entryPoint);
			} else if ( false !== strpos( $entryPoint, '/css/' ) ) {
				wp_enqueue_style($entryPoint);
			}
		}
	}
}
/**
 * Charger les assets seulement au rendu du shortcode
 */
function enqueue_shortcode_assets()
{
  $styles = [
    'bootstrap_style',
    'font_style',
    'material_style',
    'agis_style',
    'font-awesome_style',
//    'datepickerrange_style',
    'jquery-ui',
    'agis_datepickerrange_style'
  ];
  foreach ( $styles as $style ) {
    wp_enqueue_style( $style );
  }
  $scripts = [
    'mustache_script',
    'moment_script',
    'bootstrap',
    'daterange_script',
    'daterange2_script',
    'script',
    'conditionnalfield_script',
    'agis_script',
    'vuejs_script',
    'zep_script',
    'material_script',
    'jquery-ui',
    'jquery-autocomplete',
    'jquery-combobox',
      'popper',
  ];
  foreach ( $scripts as $script ) {
    wp_enqueue_script( $script );
  }
  do_action("enqueue_mc_scripts");
}

function register_shortcode($atts, $content, $shortcode_tag)
{
	$parent_source = get_option('parent_source');
	$token_type = get_option('token_type');
	$access_token = get_option('access_token');
	loadReactAssets('/assets/famille');
	$post_id = $atts['id'];
	ob_start();
	if ($parent_source != false && $token_type != false && $access_token != false) {
		$request = \Requests::get($parent_source . '/api/v1/obtenir_produit/' . $post_id, [
			'Authorization' => $token_type . ' ' . $access_token
		]);
		$forms = [];
		if ($request->status_code === 200) {
			$results = json_decode($request->body);
			$forms = $results->forms;
	?>
		<div class="form-wrapper" style="max-width: 786px">
			<!-- Circles which indicates the steps of the form: -->
			<div class="left-side" style="display: none">
				<?php
				foreach ($forms as $key => $etape) {
				?>
					<span class="step"> <?= $etape->title ?></span><br />
				<?php
				}
				?>
			</div>
			<div class="form-container">
				<?php
				if (!empty($forms)) {
					foreach ($forms as $etape) {
				?>
						<div class="tab">
							<?= $etape->content ?>
						</div>
				<?php
					}
				}
				?>
			</div>
		</div>
	<?php
		}else {
			print(__('Erreur lors de l\'obtention de la liste des formulaires du produit', 'mondialcare'));
		}
	}
	$html = ob_get_contents();
	ob_end_clean();
    enqueue_shortcode_assets();
	return $html;
}

/**
 * Save all data from configuration
 */

add_action('wp_ajax_save_configuration', 'save_configuration');

function save_configuration()
{
	foreach ($_POST as $key => $value) {
		if ($key === "action") {
			continue;
		} else {
			update_option($key, $value);
		}
	}
	update_option('parent_source_status', 'ok');
	die();
}

/**
 * Remove all configuration
 */

add_action('wp_ajax_remove_configuration', 'remove_configuration');

function remove_configuration()
{
	delete_option('parent_source_status');
	delete_option('parent_source');
	delete_option('client_id');
	delete_option('client_secret');
	delete_option('token_type');
	delete_option('access_token');
	die();
}

/**
 * add all dependance script and css to BO
 */
add_action('admin_enqueue_scripts', 'back_script_MC_plugin');
function back_script_MC_plugin()
{
	$parent_site = get_option('parent_source');
	$token_type = get_option('token_type');
	$access_token = get_option('access_token');
	wp_enqueue_style('wp-color-picker');
	wp_enqueue_style('bootstrap_style', plugins_url('assets/css/bootstrap.min.css', __FILE__), false, '4.3.1');
	wp_enqueue_style('font-awesome_style', plugins_url('assets/css/font-awesome.min.css', __FILE__), false, '4.7.0');
	wp_enqueue_style('personnalisation_style', plugins_url('assets/css/personnalisation.css', __FILE__), false, '');
	wp_enqueue_script('bootstrap_script', plugins_url('assets/js/bootstrap.min.js', __FILE__), [], '4.3.1', true);
	wp_enqueue_script('requete_script', plugins_url('assets/js/requete.js', __FILE__), ['wp-color-picker'], '', true);
	wp_localize_script('requete_script', 'saveConfiguration', [
		'parent_site' => $parent_site,
		'token_type' => $token_type,
		'access_token' => $access_token,
		'ajaxurl' => admin_url('admin-ajax.php')
	]);
}


/**
 * add all dependance script and css to FO
 */
add_action('wp_enqueue_scripts', 'script_MC_plugin', 100, 0);
function script_MC_plugin()
{

	$parent_site = get_option('parent_source');
	$token_type = get_option('token_type');
	$access_token = get_option('access_token');

	$btn_p_bg_color = get_option('mc_button_p_background_color');
	$btn_p_label_color = get_option('mc_button_p_label_color');
	$btn_p_border_color = get_option('mc_border_p_color');
	$btn_p_bg_color_focus = get_option('mc_button_p_background_color_on_focus');
	$btn_p_label_color_focus = get_option('mc_button_p_label_color_on_focus');
	$btn_p_border_color_focus = get_option('mc_border_p_color_on_focus');
	$btn_bg_color = get_option('mc_button_background_color');
	$btn_label_color = get_option('mc_button_label_color');
	$btn_border_color = get_option('mc_border_color');
	$btn_bg_color_focus = get_option('mc_button_background_color_on_focus');
	$btn_label_color_focus = get_option('mc_button_label_color_on_focus');
	$btn_border_color_focus = get_option('mc_border_color_on_focus');
	$base_color = get_option('mc_base_color');
	$custom_button = get_option('mc_custom_button');
	$primary_color = get_option('mc_primary_color');
	$secondary_color = get_option('mc_secondary_color');
	$custom_font_title = get_option('mc_custom_font_title');
	$font_title = get_option('mc_font_title');
	$custom_font_label_button = get_option('mc_custom_font_label_button');
	$font_label_button = get_option('mc_font_label_button');
	$font_label_button_size = get_option('mc_label_button_size');
	$custom_inputs = get_option('mc_custom_inputs');
	$inputs_color = get_option('mc_inputs_color');

	wp_register_style('bootstrap_style', plugins_url('assets/css/bootstrap.min.css', __FILE__), false, '4.3.1');
	//importation du font
	if ($custom_font_title === "1") {
		wp_register_style('font_style', "http://fonts.googleapis.com/css?family=" . $font_title, false, '');
	}
	if ($custom_font_label_button === "1") {
		wp_register_style('font_style', "http://fonts.googleapis.com/css?family=" . $font_label_button, false, '');
	}
	wp_register_style('material_style', plugins_url('assets/css/material-styles.css', __FILE__), false, '');
	wp_register_style('agis_style', plugins_url('assets/css/agis.css', __FILE__), false, '1.0');
	wp_register_style('font-awesome_style', plugins_url('assets/css/font-awesome.min.css', __FILE__), false, '4.7.0');
	wp_register_style('datepickerrange_style', plugins_url('assets/css/daterangepicker.css', __FILE__), false, '1.0');
	wp_register_style('agis_datepickerrange_style', plugins_url('assets/css/agis-daterangepicker.css', __FILE__), false, '1.0');
	wp_register_style("jquery-ui", "https://code.jquery.com/ui/1.12.1/themes/base/jquery-ui.css");
	if ($base_color === "1") {
		wp_add_inline_style('material_style', '
	        :root {
                --color-primary : ' . $primary_color . ';
                --color-secondary : ' . $secondary_color . ';
            }');
	}
	//utilisation du font
	if ($custom_font_title === "1") {
		wp_add_inline_style('agis_style', '
            h1,h2,h3,h4,h5 {
                font-family: ' . $font_title . ' !important;
            }');
	}
	if ($custom_font_label_button === "1") {
		wp_add_inline_style('agis_style', '
            .wpcf7-submit {
                font-family: ' . $font_label_button . '!important;
                font-size: ' . $font_label_button_size . 'px !important;
            }
        ');
	}
	if ($custom_inputs === "1") {
		wp_add_inline_style('agis_style', '
            .daterangepicker td.active, .daterangepicker td.active:hover {
                background-color: ' . $inputs_color . ' !important;
            }
		    .wpcf7 .wpcf7-form .input-field input[type="text"]:focus + label, .wpcf7-form input:not([type]):focus:not([readonly]) + label, .wpcf7-form input[type="text"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="password"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="email"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="url"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="time"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="date"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="datetime"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="datetime-local"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="tel"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="number"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form input[type="search"]:not(.browser-default):focus:not([readonly]) + label, .wpcf7-form textarea.materialize-textarea:focus:not([readonly]) + label, .wpcf7-form .dropdown-content li > a, .wpcf7-form .dropdown-content li > span {
                color: ' . $inputs_color . ' !important;
            }
            .wpcf7-form [type="checkbox"]:checked + label::before {
                border-right: 2px solid ' . $inputs_color . ' ;
                border-bottom: 2px solid ' . $inputs_color . ' ;
            }
		    .wpcf7-form input:not([type]):focus:not([readonly]), .wpcf7-form input[type="text"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="password"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="email"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="url"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="time"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="date"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="datetime"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="datetime-local"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="tel"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="number"]:not(.browser-default):focus:not([readonly]), .wpcf7-form input[type="search"]:not(.browser-default):focus:not([readonly]), .wpcf7-form textarea.materialize-textarea:focus:not([readonly]) {
                border-bottom: 1px solid ' . $inputs_color . ' !important;
                box-shadow: 0 1px 0 0 ' . $inputs_color . ' !important;
            }
            .wpcf7-form [type="radio"]:checked + label::after, .wpcf7-form [type="radio"].with-gap:checked + label::after, .wpcf7-form .switch label input[type="checkbox"]:checked + .lever::after {
                background-color: ' . $inputs_color . ' !important;
            }
            .wpcf7-form [type="radio"]:checked + label::after, .wpcf7-form [type="radio"].with-gap:checked + label::before, .wpcf7-form [type="radio"].with-gap:checked + label::after {
                border: 2px solid ' . $inputs_color . ' !important;
            }');
	}

	if ($custom_button === "1") {
		wp_add_inline_style('material_style', '
            :root {
                --btn-p-bg-color : ' . $btn_p_bg_color . ';
                --btn-p-label-color : ' . $btn_p_label_color . ';
                --btn-p-border-color : ' . $btn_p_border_color . ';
                --btn-p-bg-color-focus : ' . $btn_p_bg_color_focus . ';
                --btn-p-label-color-focus : ' . $btn_p_label_color_focus . ';
                --btn-p-border-color-focus : ' . $btn_p_border_color_focus . ';
                --btn-bg-color : ' . $btn_bg_color . ';
                --btn-label-color : ' . $btn_label_color . ';
                --btn-border-color : ' . $btn_border_color . ';
                --btn-bg-color-focus : ' . $btn_bg_color_focus . ';
                --btn-label-color-focus : ' . $btn_label_color_focus . ';
                --btn-border-color-focus : ' . $btn_border_color_focus . ';
            }
            #prevBtn {
                background-color: var(--btn-p-bg-color) !important;
                color: var(--btn-p-label-color) !important;
                border-color: var(--btn-p-border-color) !important;
            }
            #prevBtn:hover {
                background-color: var(--btn-p-bg-color-focus) !important;
                color: var(--btn-p-label-color-focus) !important;
                border-color: var(--btn-p-border-color-focus) !important;
            }
            #nextBtn {
                background-color: var(--btn-bg-color) !important;
                color: var(--btn-label-color) !important;
                border-color: var(--btn-border-color) !important;
            }
            #nextBtn:hover {
                background-color: var(--btn-bg-color-focus) !important;
                color: var(--btn-label-color-focus) !important;
                border-color: var(--btn-border-color-focus) !important;
            }
        ');
	}

	wp_register_script('mustache_script', plugins_url('assets/js/mustache.min.js', __FILE__), [], '', true);
	wp_register_script('moment_script', plugins_url('assets/js/moment.min.js', __FILE__), [], '', true);
	wp_register_script('bootstrap', plugins_url('assets/js/bootstrap.min.js', __FILE__), [], '4.3.1', true);
	wp_register_script('daterange_script', plugins_url('assets/js/daterangepicker.js', __FILE__), [], '1.0', true);
	wp_register_script('daterange2_script', plugins_url('assets/js/Date-Range-Picker.js', __FILE__), [], '1.0', true);
	wp_register_script('script', plugins_url('assets/js/scripts.js', __FILE__), [], '1.0', true);
	wp_register_script('conditionnalfield_script', plugins_url('assets/js/conditionnal_field.js', __FILE__), [], '', true);
	wp_register_script('agis_script', plugins_url('assets/js/agis.js', __FILE__), [], '1.0', true);
	wp_register_script('vuejs_script', 'https://cdn.jsdelivr.net/npm/vue@2.6.10/dist/vue.js', [], '', true);
	wp_register_script('zep_script', plugins_url('assets/js/zep.js', __FILE__), [], '', true);
	wp_register_script('material_script', plugins_url('assets/js/material.js', __FILE__), [], '', true);
	wp_register_script("jquery-ui", "https://code.jquery.com/ui/1.12.1/jquery-ui.js",	'', '', false);
	wp_register_script("jquery-autocomplete", plugins_url('assets/js/jquery.autocomplete.min.js', __FILE__), ["jquery", 'jquery-ui'], '', true);
	wp_register_script("jquery-combobox",	plugins_url('assets/js/jquery.combobox.v2.js', __FILE__), ["jquery-autocomplete"], '', true);
    wp_register_script("popper",	'https://unpkg.com/@popperjs/core@2.11.4/dist/umd/popper.min.js', [], '', false);
	wp_add_inline_script('script', 'var parent_site="' . $parent_site . '"', 'before');
	wp_localize_script('script', 'ajaxToParent', [
		'parent_site' => $parent_site,
		'token_type' => $token_type,
		'access_token' => $access_token
	]);
	wp_localize_script('jquery-combobox', 'ajaxToParent', [
		'parent_site' => $parent_site,
		'token_type' => $token_type,
		'access_token' => $access_token
	]);
	wp_localize_script('daterange2_script', 'ajaxToParent', [
		'parent_site' => $parent_site,
		'token_type' => $token_type,
		'access_token' => $access_token
	]);
	wp_localize_script('zep_script', 'ajaxToParent', [
		'parent_site' => $parent_site,
		'token_type' => $token_type,
		'access_token' => $access_token
	]);
}
function mondialcare_header()
{
	?>
	<div class='wrap'>
		<div class="grid">
			<div class="grid-item">
				<img src="<?= plugins_url("assets/Logo MondialCare Horizontal.png", __FILE__) ?>" />
			</div>
		</div>
	</div>
<?php
}
function mondialcare_footer()
{
?>
	<style>
		.blue {
			color: #339999;
		}
	</style>
	<div class='wrap' style="clear: both;padding-top: 1rem;">
		<hr />
		<div class="grid">
			<div class="grid-item" align='right'>
				<ul>
					<li>Contactez notre service technique au <span class='blue'><a href="tel:+33(0)182835622">+33(0)182835622</a></span></li>
					<li>Contactez le service commercial au <span class='blue'><a href="tel:+33(0)182835626">+33(0)182835626</a></span></li>
					<li>Vous pouvez aussi nous joindre par e-mail : <span class='blue'><a href="mailto:informatique@agis-group.com">informatique@agis-group.com</a></span></li>
				</ul>
			</div>
		</div>
	</div>
<?php
}
add_action('plugins_loaded', function () {
	MC_Plugin::get_instance();
});
